/*
 * Copyright (C) 2010 Jacob Kjome
 * 
 * This file is part of XMLC.
 *
 * XMLC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation, either version 2.1
 * of the License, or (at your option) any later version.
 *
 * XMLC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with XMLC.  If not,
 * see <http://www.gnu.org/licenses/>.
 */
var _style_cookie_name = 'style-switch';

function setActiveStyleSheet(title) {
    if (!title || !document.styleSheets) return;
    var ss = document.styleSheets;
    for (var i = 0; i < ss.length; i++) {
        var a = ss[i];
        if (a.title) a.disabled = !(a.title == title);
    }
}

function toggleEnableStyleSheet(title) {
    if (!title || !document.styleSheets) return;
    var ss = document.styleSheets;
    for (var i = 0; i < ss.length; i++) {
        var a = ss[i];
        if (a.title && a.title == title) a.disabled = !a.disabled;
    }
}

function writeStyleSwitcher(locationNode) {
    if (!locationNode || !document.styleSheets) return;
    var ss = document.styleSheets;

    var wrapper = document.createElement('div');
    wrapper.id = 'style-switch-wrapper';

    //IE doesn't work well with dynamically added input elements, at least
    //enclosed in label elements, when using the w3c dom, so write as innerHTML
    if (document.all) {        
        var store = new Array();
        store.push('<span>Use Style:</span> ');
        for (var i = 0; i < ss.length; i++ ) {
        	var a = ss[i];
            if (!a.title) continue;
            store.push('<label for="');store.push(a.title);store.push('">');store.push(a.title);
            store.push('<input id="');store.push(a.title);store.push('" type="radio" name="style-switch" onclick="');store.push("setActiveStyleSheet('");store.push(a.title);store.push('\');" />');
            store.push('</label>');
        }
        var writestring = store.join('');
        wrapper.innerHTML = writestring;
    }
    else {
        var divEl = document.createElement('div');
        var spanEl = document.createElement('span');
        spanEl.appendChild(document.createTextNode('Use Style: '));
        wrapper.appendChild(spanEl);
        var labelEl = document.createElement('label');
        var inputEl = document.createElement('input');
        inputEl.setAttribute('type', 'radio');
        inputEl.setAttribute('name', 'style-switch');
        for (var i = 0; i < ss.length; i++ ) {
        	var a = ss[i];
            if (!a.title) continue;
            var aLabel = labelEl.cloneNode(false);
            aLabel.setAttribute('for', a.title);
            aLabel.appendChild(document.createTextNode(a.title));
            var aInput = inputEl.cloneNode(false);
            aInput.id = a.title;
            aInput.setAttribute("onclick", "setActiveStyleSheet('" + a.title + "');");
            aLabel.appendChild(aInput);
            wrapper.appendChild(aLabel);
            wrapper.appendChild(document.createTextNode(' '));
        }
        wrapper.removeChild(wrapper.lastChild);
    }
    locationNode.insertBefore(wrapper, locationNode.firstChild);
}


function getActiveStyleSheet() {
    if (!document.styleSheets) return null;
    var ss = document.styleSheets;
    for (var i = 0; i < ss.length; i++) {
    	var a = ss[i];
        if (a.title && !a.disabled) return a.title;
    }
    return null;
}

function getPreferredStyleSheet() {
    if (!document.styleSheets) return null;
    var ss = document.styleSheets;
    for (var i = 0; i < ss.length; i++) {
        var a = ss[i];
        if (a.title && (a.owningElement ? a.owningElement : a.ownerNode).rel.indexOf('alt') == -1) return a.title;
    }
    return null;
}

function createCookie(name, value, days) {
    if (days) {
        var date = new Date();
        date.setTime(date.getTime() + (days * 24 * 60 * 60 * 1000));
        var expires = '; expires=' + date.toGMTString();
    }
    else expires = '';
    document.cookie = name + '=' + value + expires + '; path=/';
}

function readCookie(name) {
	var nameEQ = name + '=';
    var cs = document.cookie.split(';');
    for (var i = 0; i < cs.length; i++) {
        var c = cs[i];
        while (c.charAt(0) == ' ') c = c.substring(1, c.length);
        if (c.indexOf(nameEQ) == 0) return c.substring(nameEQ.length, c.length);
    }
    return null;
}

function eraseCookie(name) {
	createCookie(name, '', -1);
}
