/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: ElementClassMappings.java,v 1.2 2005-01-26 08:29:24 jkjome Exp $
 */

package tests.org.enhydra.xml.xhtml.dominfo;

import java.io.File;
import java.io.FilenameFilter;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Set;
import java.util.StringTokenizer;

import org.enhydra.error.FatalExceptionError;
import org.enhydra.xml.xhtml.dom.XHTMLAbbrElement;
import org.enhydra.xml.xhtml.dom.XHTMLAcronymElement;
import org.enhydra.xml.xhtml.dom.XHTMLAddressElement;
import org.enhydra.xml.xhtml.dom.XHTMLAnchorElement;
import org.enhydra.xml.xhtml.dom.XHTMLAppletElement;
import org.enhydra.xml.xhtml.dom.XHTMLAreaElement;
import org.enhydra.xml.xhtml.dom.XHTMLBElement;
import org.enhydra.xml.xhtml.dom.XHTMLBRElement;
import org.enhydra.xml.xhtml.dom.XHTMLBaseElement;
import org.enhydra.xml.xhtml.dom.XHTMLBaseFontElement;
import org.enhydra.xml.xhtml.dom.XHTMLBdoElement;
import org.enhydra.xml.xhtml.dom.XHTMLBigElement;
import org.enhydra.xml.xhtml.dom.XHTMLBodyElement;
import org.enhydra.xml.xhtml.dom.XHTMLButtonElement;
import org.enhydra.xml.xhtml.dom.XHTMLCenterElement;
import org.enhydra.xml.xhtml.dom.XHTMLCiteElement;
import org.enhydra.xml.xhtml.dom.XHTMLCodeElement;
import org.enhydra.xml.xhtml.dom.XHTMLDListElement;
import org.enhydra.xml.xhtml.dom.XHTMLDdElement;
import org.enhydra.xml.xhtml.dom.XHTMLDfnElement;
import org.enhydra.xml.xhtml.dom.XHTMLDirectoryElement;
import org.enhydra.xml.xhtml.dom.XHTMLDivElement;
import org.enhydra.xml.xhtml.dom.XHTMLDtElement;
import org.enhydra.xml.xhtml.dom.XHTMLElement;
import org.enhydra.xml.xhtml.dom.XHTMLEmElement;
import org.enhydra.xml.xhtml.dom.XHTMLFieldSetElement;
import org.enhydra.xml.xhtml.dom.XHTMLFontElement;
import org.enhydra.xml.xhtml.dom.XHTMLFormElement;
import org.enhydra.xml.xhtml.dom.XHTMLFrameElement;
import org.enhydra.xml.xhtml.dom.XHTMLFrameSetElement;
import org.enhydra.xml.xhtml.dom.XHTMLHRElement;
import org.enhydra.xml.xhtml.dom.XHTMLHeadElement;
import org.enhydra.xml.xhtml.dom.XHTMLHeadingElement;
import org.enhydra.xml.xhtml.dom.XHTMLHtmlElement;
import org.enhydra.xml.xhtml.dom.XHTMLIElement;
import org.enhydra.xml.xhtml.dom.XHTMLIFrameElement;
import org.enhydra.xml.xhtml.dom.XHTMLImageElement;
import org.enhydra.xml.xhtml.dom.XHTMLInputElement;
import org.enhydra.xml.xhtml.dom.XHTMLIsIndexElement;
import org.enhydra.xml.xhtml.dom.XHTMLKbdElement;
import org.enhydra.xml.xhtml.dom.XHTMLLIElement;
import org.enhydra.xml.xhtml.dom.XHTMLLabelElement;
import org.enhydra.xml.xhtml.dom.XHTMLLegendElement;
import org.enhydra.xml.xhtml.dom.XHTMLLinkElement;
import org.enhydra.xml.xhtml.dom.XHTMLMapElement;
import org.enhydra.xml.xhtml.dom.XHTMLMenuElement;
import org.enhydra.xml.xhtml.dom.XHTMLMetaElement;
import org.enhydra.xml.xhtml.dom.XHTMLModElement;
import org.enhydra.xml.xhtml.dom.XHTMLNoframesElement;
import org.enhydra.xml.xhtml.dom.XHTMLNoscriptElement;
import org.enhydra.xml.xhtml.dom.XHTMLOListElement;
import org.enhydra.xml.xhtml.dom.XHTMLObjectElement;
import org.enhydra.xml.xhtml.dom.XHTMLOptGroupElement;
import org.enhydra.xml.xhtml.dom.XHTMLOptionElement;
import org.enhydra.xml.xhtml.dom.XHTMLParagraphElement;
import org.enhydra.xml.xhtml.dom.XHTMLParamElement;
import org.enhydra.xml.xhtml.dom.XHTMLPreElement;
import org.enhydra.xml.xhtml.dom.XHTMLQuoteElement;
import org.enhydra.xml.xhtml.dom.XHTMLSElement;
import org.enhydra.xml.xhtml.dom.XHTMLSampElement;
import org.enhydra.xml.xhtml.dom.XHTMLScriptElement;
import org.enhydra.xml.xhtml.dom.XHTMLSelectElement;
import org.enhydra.xml.xhtml.dom.XHTMLSmallElement;
import org.enhydra.xml.xhtml.dom.XHTMLSpanElement;
import org.enhydra.xml.xhtml.dom.XHTMLStrikeElement;
import org.enhydra.xml.xhtml.dom.XHTMLStrongElement;
import org.enhydra.xml.xhtml.dom.XHTMLStyleElement;
import org.enhydra.xml.xhtml.dom.XHTMLSubElement;
import org.enhydra.xml.xhtml.dom.XHTMLSupElement;
import org.enhydra.xml.xhtml.dom.XHTMLTableCaptionElement;
import org.enhydra.xml.xhtml.dom.XHTMLTableCellElement;
import org.enhydra.xml.xhtml.dom.XHTMLTableColElement;
import org.enhydra.xml.xhtml.dom.XHTMLTableElement;
import org.enhydra.xml.xhtml.dom.XHTMLTableRowElement;
import org.enhydra.xml.xhtml.dom.XHTMLTbodyElement;
import org.enhydra.xml.xhtml.dom.XHTMLTextAreaElement;
import org.enhydra.xml.xhtml.dom.XHTMLTfootElement;
import org.enhydra.xml.xhtml.dom.XHTMLTheadElement;
import org.enhydra.xml.xhtml.dom.XHTMLTitleElement;
import org.enhydra.xml.xhtml.dom.XHTMLTtElement;
import org.enhydra.xml.xhtml.dom.XHTMLUElement;
import org.enhydra.xml.xhtml.dom.XHTMLUListElement;
import org.enhydra.xml.xhtml.dom.XHTMLVarElement;
import org.enhydra.xml.xhtml.dom.xerces.XHTMLElementImpl;
import org.w3c.dom.html.HTMLAnchorElement;
import org.w3c.dom.html.HTMLAppletElement;
import org.w3c.dom.html.HTMLAreaElement;
import org.w3c.dom.html.HTMLBRElement;
import org.w3c.dom.html.HTMLBaseElement;
import org.w3c.dom.html.HTMLBaseFontElement;
import org.w3c.dom.html.HTMLBodyElement;
import org.w3c.dom.html.HTMLButtonElement;
import org.w3c.dom.html.HTMLDListElement;
import org.w3c.dom.html.HTMLDirectoryElement;
import org.w3c.dom.html.HTMLDivElement;
import org.w3c.dom.html.HTMLElement;
import org.w3c.dom.html.HTMLFieldSetElement;
import org.w3c.dom.html.HTMLFontElement;
import org.w3c.dom.html.HTMLFormElement;
import org.w3c.dom.html.HTMLFrameElement;
import org.w3c.dom.html.HTMLFrameSetElement;
import org.w3c.dom.html.HTMLHRElement;
import org.w3c.dom.html.HTMLHeadElement;
import org.w3c.dom.html.HTMLHeadingElement;
import org.w3c.dom.html.HTMLHtmlElement;
import org.w3c.dom.html.HTMLIFrameElement;
import org.w3c.dom.html.HTMLImageElement;
import org.w3c.dom.html.HTMLInputElement;
import org.w3c.dom.html.HTMLIsIndexElement;
import org.w3c.dom.html.HTMLLIElement;
import org.w3c.dom.html.HTMLLabelElement;
import org.w3c.dom.html.HTMLLegendElement;
import org.w3c.dom.html.HTMLLinkElement;
import org.w3c.dom.html.HTMLMapElement;
import org.w3c.dom.html.HTMLMenuElement;
import org.w3c.dom.html.HTMLMetaElement;
import org.w3c.dom.html.HTMLModElement;
import org.w3c.dom.html.HTMLOListElement;
import org.w3c.dom.html.HTMLObjectElement;
import org.w3c.dom.html.HTMLOptGroupElement;
import org.w3c.dom.html.HTMLOptionElement;
import org.w3c.dom.html.HTMLParagraphElement;
import org.w3c.dom.html.HTMLParamElement;
import org.w3c.dom.html.HTMLPreElement;
import org.w3c.dom.html.HTMLQuoteElement;
import org.w3c.dom.html.HTMLScriptElement;
import org.w3c.dom.html.HTMLSelectElement;
import org.w3c.dom.html.HTMLStyleElement;
import org.w3c.dom.html.HTMLTableCaptionElement;
import org.w3c.dom.html.HTMLTableCellElement;
import org.w3c.dom.html.HTMLTableColElement;
import org.w3c.dom.html.HTMLTableElement;
import org.w3c.dom.html.HTMLTableRowElement;
import org.w3c.dom.html.HTMLTableSectionElement;
import org.w3c.dom.html.HTMLTextAreaElement;
import org.w3c.dom.html.HTMLTitleElement;
import org.w3c.dom.html.HTMLUListElement;

/**
 * Tables containing the element-class mappings for XHTML and HTML.
 * Also has routines to find the list of classes that are available.
 */
class ElementClassMappings {
    /**
     * Table of element name to interface mappings.
     */
    private static HashMap fXHTMLInterfaceMap = new HashMap();
    private static HashMap fXHTMLClassMap = new HashMap();
    private static HashMap fHTMLInterfaceMap = new HashMap();


    /**
     * Add an XHTML interface and implementation to the map.
     */
    private static void xhtmlAdd(String tag,
                                 Class element) {
        fXHTMLInterfaceMap.put(tag.toLowerCase(), element);

        // Construct the interface name and get the class
        String qualName = element.getName();
        String pkgName = qualName.substring(0, qualName.lastIndexOf('.'));
        String unqualName = qualName.substring(qualName.lastIndexOf('.')+1);
        String implName = pkgName + ".xerces." + unqualName + "Impl";
        try {
            fXHTMLClassMap.put(tag.toLowerCase(), Class.forName(implName));
        } catch (ClassNotFoundException except) {
            throw new FatalExceptionError(except);
        }
    }

    /**
     * Build table for XHTML.  This code taken from
     * org.enhydra.xml.xhtml.dom.xerces.XHTMLDocumentImpl.
     */
    private static void buildXHTMLTable() {
        xhtmlAdd("basefont", XHTMLBaseFontElement.class);
        xhtmlAdd("button", XHTMLButtonElement.class);
        xhtmlAdd("textarea", XHTMLTextAreaElement.class);
        xhtmlAdd("em", XHTMLEmElement.class);
        xhtmlAdd("small", XHTMLSmallElement.class);
        xhtmlAdd("area", XHTMLAreaElement.class);
        xhtmlAdd("noframes", XHTMLNoframesElement.class);
        xhtmlAdd("bdo", XHTMLBdoElement.class);
        xhtmlAdd("form", XHTMLFormElement.class);
        xhtmlAdd("link", XHTMLLinkElement.class);
        xhtmlAdd("label", XHTMLLabelElement.class);
        xhtmlAdd("dt", XHTMLDtElement.class);
        xhtmlAdd("span", XHTMLSpanElement.class);
        xhtmlAdd("isindex", XHTMLIsIndexElement.class);
        xhtmlAdd("title", XHTMLTitleElement.class);
        xhtmlAdd("strong", XHTMLStrongElement.class);
        xhtmlAdd("script", XHTMLScriptElement.class);
        xhtmlAdd("div", XHTMLDivElement.class);
        xhtmlAdd("dl", XHTMLDListElement.class);
        xhtmlAdd("blockquote", XHTMLQuoteElement.class);
        xhtmlAdd("kbd", XHTMLKbdElement.class);
        xhtmlAdd("menu", XHTMLMenuElement.class);
        xhtmlAdd("body", XHTMLBodyElement.class);
        xhtmlAdd("dir", XHTMLDirectoryElement.class);
        xhtmlAdd("ins", XHTMLModElement.class);
        xhtmlAdd("map", XHTMLMapElement.class);
        xhtmlAdd("dd", XHTMLDdElement.class);
        xhtmlAdd("fieldset", XHTMLFieldSetElement.class);
        xhtmlAdd("head", XHTMLHeadElement.class);
        xhtmlAdd("col", XHTMLTableColElement.class);
        xhtmlAdd("base", XHTMLBaseElement.class);
        xhtmlAdd("big", XHTMLBigElement.class);
        xhtmlAdd("meta", XHTMLMetaElement.class);
        xhtmlAdd("code", XHTMLCodeElement.class);
        xhtmlAdd("tbody", XHTMLTbodyElement.class);
        xhtmlAdd("option", XHTMLOptionElement.class);
        xhtmlAdd("u", XHTMLUElement.class);
        xhtmlAdd("s", XHTMLSElement.class);
        xhtmlAdd("q", XHTMLQuoteElement.class);
        xhtmlAdd("p", XHTMLParagraphElement.class);
        xhtmlAdd("ol", XHTMLOListElement.class);
        xhtmlAdd("thead", XHTMLTheadElement.class);
        xhtmlAdd("ul", XHTMLUListElement.class);
        xhtmlAdd("i", XHTMLIElement.class);
        xhtmlAdd("pre", XHTMLPreElement.class);
        xhtmlAdd("optgroup", XHTMLOptGroupElement.class);
        xhtmlAdd("img", XHTMLImageElement.class);
        xhtmlAdd("caption", XHTMLTableCaptionElement.class);
        xhtmlAdd("b", XHTMLBElement.class);
        xhtmlAdd("a", XHTMLAnchorElement.class);
        xhtmlAdd("frame", XHTMLFrameElement.class);
        xhtmlAdd("br", XHTMLBRElement.class);
        xhtmlAdd("style", XHTMLStyleElement.class);
        xhtmlAdd("hr", XHTMLHRElement.class);
        xhtmlAdd("param", XHTMLParamElement.class);
        xhtmlAdd("table", XHTMLTableElement.class);
        xhtmlAdd("applet", XHTMLAppletElement.class);
        xhtmlAdd("tt", XHTMLTtElement.class);
        xhtmlAdd("tr", XHTMLTableRowElement.class);
        xhtmlAdd("th", XHTMLTableCellElement.class);
        xhtmlAdd("center", XHTMLCenterElement.class);
        xhtmlAdd("td", XHTMLTableCellElement.class);
        xhtmlAdd("samp", XHTMLSampElement.class);
        xhtmlAdd("tfoot", XHTMLTfootElement.class);
        xhtmlAdd("font", XHTMLFontElement.class);
        xhtmlAdd("dfn", XHTMLDfnElement.class);
        xhtmlAdd("noscript", XHTMLNoscriptElement.class);
        xhtmlAdd("object", XHTMLObjectElement.class);
        xhtmlAdd("colgroup", XHTMLTableColElement.class);
        xhtmlAdd("sup", XHTMLSupElement.class);
        xhtmlAdd("html", XHTMLHtmlElement.class);
        xhtmlAdd("h6", XHTMLHeadingElement.class);
        xhtmlAdd("h5", XHTMLHeadingElement.class);
        xhtmlAdd("h4", XHTMLHeadingElement.class);
        xhtmlAdd("h3", XHTMLHeadingElement.class);
        xhtmlAdd("frameset", XHTMLFrameSetElement.class);
        xhtmlAdd("h2", XHTMLHeadingElement.class);
        xhtmlAdd("h1", XHTMLHeadingElement.class);
        xhtmlAdd("iframe", XHTMLIFrameElement.class);
        xhtmlAdd("strike", XHTMLStrikeElement.class);
        xhtmlAdd("sub", XHTMLSubElement.class);
        xhtmlAdd("acronym", XHTMLAcronymElement.class);
        xhtmlAdd("select", XHTMLSelectElement.class);
        xhtmlAdd("del", XHTMLModElement.class);
        xhtmlAdd("li", XHTMLLIElement.class);
        xhtmlAdd("cite", XHTMLCiteElement.class);
        xhtmlAdd("var", XHTMLVarElement.class);
        xhtmlAdd("legend", XHTMLLegendElement.class);
        xhtmlAdd("abbr", XHTMLAbbrElement.class);
        xhtmlAdd("input", XHTMLInputElement.class);
        xhtmlAdd("address", XHTMLAddressElement.class);
    }

    /**
     * Add an HTML interface to the map.
     */
    private static void htmlAdd(String tag,
                                Class element) {
        fHTMLInterfaceMap.put(tag.toLowerCase(), element);
    }
    
    /**
     * Build table for HTML.  This code taken from
     * org.apache.html.dom.HTMLDocumentImpl.
     */
    private static void buildHTMLTable() {
        htmlAdd("A", HTMLAnchorElement.class);
        htmlAdd("APPLET", HTMLAppletElement.class);
        htmlAdd("AREA", HTMLAreaElement.class);
        htmlAdd("BASE", HTMLBaseElement.class);
        htmlAdd("BASEFONT", HTMLBaseFontElement.class);
        htmlAdd("BLOCKQUOTE", HTMLQuoteElement.class);
        htmlAdd("BODY", HTMLBodyElement.class);
        htmlAdd("BR", HTMLBRElement.class);
        htmlAdd("BUTTON", HTMLButtonElement.class);
        htmlAdd("DEL", HTMLModElement.class);
        htmlAdd("DIR", HTMLDirectoryElement.class);
        htmlAdd("DIV", HTMLDivElement.class);
        htmlAdd("DL", HTMLDListElement.class);
        htmlAdd("FIELDSET", HTMLFieldSetElement.class);
        htmlAdd("FONT", HTMLFontElement.class);
        htmlAdd("FORM", HTMLFormElement.class);
        htmlAdd("FRAME", HTMLFrameElement.class);
        htmlAdd("FRAMESET", HTMLFrameSetElement.class);
        htmlAdd("HEAD", HTMLHeadElement.class);
        htmlAdd("H1", HTMLHeadingElement.class);
        htmlAdd("H2", HTMLHeadingElement.class);
        htmlAdd("H3", HTMLHeadingElement.class);
        htmlAdd("H4", HTMLHeadingElement.class);
        htmlAdd("H5", HTMLHeadingElement.class);
        htmlAdd("H6", HTMLHeadingElement.class);
        htmlAdd("HR", HTMLHRElement.class);
        htmlAdd("HTML", HTMLHtmlElement.class);
        htmlAdd("IFRAME", HTMLIFrameElement.class);
        htmlAdd("IMG", HTMLImageElement.class);
        htmlAdd("INPUT", HTMLInputElement.class);
        htmlAdd("INS", HTMLModElement.class);
        htmlAdd("ISINDEX", HTMLIsIndexElement.class);
        htmlAdd("LABEL", HTMLLabelElement.class);
        htmlAdd("LEGEND", HTMLLegendElement.class);
        htmlAdd("LI", HTMLLIElement.class);
        htmlAdd("LINK", HTMLLinkElement.class);
        htmlAdd("MAP", HTMLMapElement.class);
        htmlAdd("MENU", HTMLMenuElement.class);
        htmlAdd("META", HTMLMetaElement.class);
        htmlAdd("OBJECT", HTMLObjectElement.class);
        htmlAdd("OL", HTMLOListElement.class);
        htmlAdd("OPTGROUP", HTMLOptGroupElement.class);
        htmlAdd("OPTION", HTMLOptionElement.class);
        htmlAdd("P", HTMLParagraphElement.class);
        htmlAdd("PARAM", HTMLParamElement.class);
        htmlAdd("PRE", HTMLPreElement.class);
        htmlAdd("Q", HTMLQuoteElement.class);
        htmlAdd("SCRIPT", HTMLScriptElement.class);
        htmlAdd("SELECT", HTMLSelectElement.class);
        htmlAdd("STYLE", HTMLStyleElement.class);
        htmlAdd("TABLE", HTMLTableElement.class);
        htmlAdd("CAPTION", HTMLTableCaptionElement.class);
        htmlAdd("TD", HTMLTableCellElement.class);
        htmlAdd("TH", HTMLTableCellElement.class);
        htmlAdd("COL", HTMLTableColElement.class);
        htmlAdd("COLGROUP", HTMLTableColElement.class);
        htmlAdd("TR", HTMLTableRowElement.class);
        htmlAdd("TBODY", HTMLTableSectionElement.class);
        htmlAdd("THEAD", HTMLTableSectionElement.class);
        htmlAdd("TFOOT", HTMLTableSectionElement.class);
        htmlAdd("TEXTAREA", HTMLTextAreaElement.class);
        htmlAdd("TITLE", HTMLTitleElement.class);
        htmlAdd("UL", HTMLUListElement.class);
    }

    /**
     * Class initializer.
     */
    static {
        buildXHTMLTable();
        buildHTMLTable();
    }

    /**
     * Get a XHTML interface given a tag name.
     */
    public static Class getXHTMLInterface(String tag) {
        Class cl = (Class)fXHTMLInterfaceMap.get(tag.toLowerCase());
        if (cl == null) {
            cl = XHTMLElement.class;
        }
        return cl;
    }

    /**
     * Get a XHTML implementation given a tag name.
     */
    public static Class getXHTMLImplementation(String tag) {
        Class cl = (Class)fXHTMLClassMap.get(tag.toLowerCase());
        if (cl == null) {
            cl = XHTMLElementImpl.class;
        }
        return cl;
    }

    /**
     * Get a HTML interface given a tag name.
     */
    public static Class getHTMLInterface(String tag) {
        Class cl = (Class)fHTMLInterfaceMap.get(tag.toLowerCase());
        if (cl == null) {
            cl = HTMLElement.class;
        }
        return cl;
    }

    /**
     * Search a class path entry for all class files in a package that containg
     * the string `Element'.
     */
    private static void searchClassPathEntry(String pathEntry,
                                             String pkgName,
                                             String pkgDir,
                                             Set classNames) {
        final String suffix = ".class";
        File dirFile = new File(pathEntry + "/" + pkgDir);
        File[] files = dirFile.listFiles(new FilenameFilter() {
                public boolean accept(File dir, String name) {
                    // Is the name in the form ".*Element*.class"?
                    return name.regionMatches(name.length()-suffix.length(), suffix, 0, suffix.length())
                        && (name.indexOf("Element") >= 0);
                }
            });
        // Convert to class names
        if (files != null) {
            for (int i = 0; i < files.length; i++) {
                String name = files[i].getName();
                classNames.add(pkgName + "." + name.substring(0, name.length()-suffix.length()));
            }
        }
    }

    /**
     * Search the CLASSPATH for all classes in the specified package
     * that containg the string `Element'.
     * FIXME: doesn't currently handle jars
     * @returns a list of class names.
     */
    public static String[] getPackageElementClasses(String pkgName) {
        HashSet classNames = new HashSet();
        String pkgDir = pkgName.replace('.', '/');
        String path = System.getProperty("java.class.path");
        StringTokenizer tokenizer = new StringTokenizer(path, File.pathSeparator);
        while (tokenizer.hasMoreTokens()) {
            searchClassPathEntry((String)tokenizer.nextToken(), pkgName, pkgDir, classNames);
        }
        return (String[])classNames.toArray(new String[classNames.size()]);
    }

}
