/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: Encoding.java,v 1.2 2005-01-26 08:29:24 jkjome Exp $
 */
package org.enhydra.xml.io;



/**
 * Do mapping between Java and XML encodings.
 * @deprecated Use org.enhydra.xml.io.Encodings
 * @see org.enhydra.xml.io.Encodings
 */
public class Encoding {
    /**
     * Disallow instantiation.
     */
    private Encoding() {
    }

    /**
     * Determine if an encoding is a supported Java encoding.
     */
    public static boolean isValidJavaEncoding(String javaEncoding) {
        return Encodings.getEncodings().isValid(javaEncoding);
    }

    /**
     * Convert a Java encoding to a XML encoding.
     * @return the XML encoding, or null if the encoding couldn't
     *  be converted.
     */
    public static String java2XmlEncoding(String javaEncoding) {
        return Encodings.getEncodings().getMIMEPreferred(javaEncoding);
    }

    /**
     * Determine if an encoding is a supported XML encoding.
     */
    public static boolean isValidXMLEncoding(String xmlEncoding) {
        return Encodings.getEncodings().isValid(xmlEncoding);
    }

    /**
     * Convert a XML encoding to a Java encoding.
     * @return the Java encoding, or null if the encoding couldn't
     *  be converted.
     */
    public static String xml2JavaEncoding(String xmlEncoding) {
        return Encodings.getEncodings().getName(xmlEncoding);
    }

    /**
     * Get the maximum value of an unicode character in an XML encoding.
     * This checks for 7 and 8 bit encodings; everyting else is considered
     * to require 16 bits.  Logic for this method stolen from
     * org.apache.xml.serialize.OutputFormat.
     */
    public static int getMaxCharacterValue(String xmlEncoding) {
        return Encodings.getEncodings().getMaxCharacterValue(xmlEncoding);
    }
}
