/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: PreFormattedText.java,v 1.1.1.1 2003-03-10 16:36:17 taweili Exp $
 */

package org.enhydra.xml.io;

/**
 * Interface that nodes of a document must implement if they have formatted
 * text. Formatted text can be available on:
 * <UL>
 * <LI> Text nodes - Formatted text is a string with entity references already
 *      expanded.
 * <LI> Element nodes - Formatted text is the opening tag, with entity
 *      references in attribute-values expanded.
 * </UL>
 */
public interface PreFormattedText {
    /**
     * Get the pre-formatted text for a node.
     *
     * @return The pre-formatted text string, or null if no formatted
     *  text is available or the text is no longer valid.
     */
    public String getPreFormattedText();

    // FIXME: set might not be needed (DOM specific).
    /**
     * Set the pre-formatted text for a node.
     */
    public void setPreFormattedText(String text);
}
