/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: LazyDOMImplementation.java,v 1.5 2008-04-13 19:47:46 jkjome Exp $
 */

package org.enhydra.xml.lazydom;

import org.apache.xerces.dom.DOMImplementationImpl;
import org.apache.xerces.dom.DOMMessageFormatter;
import org.w3c.dom.DOMException;
import org.w3c.dom.DOMImplementation;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentType;
import org.w3c.dom.Element;

/**
 * DOMImplementation method for the LazyDOM.

 * @see org.w3c.dom.DOMImplementation
 */
public class LazyDOMImplementation extends DOMImplementationImpl {
    /** DOM implementation singleton. */
    private static final LazyDOMImplementation fSingleton = new LazyDOMImplementation();

    /**
     * Obtain and return the singleton shared object.
     */
    public static DOMImplementation getDOMImplementation() {
        return fSingleton;
    }  

    /**
     * Create a new LazyDocument
     * @see org.w3c.dom.DOMImplementation#createDocument
     */
    public Document createDocument(String namespaceURI, 
                                   String qualifiedName, 
                                   DocumentType doctype) throws DOMException {
        if (doctype != null && doctype.getOwnerDocument() != null) {
            throw new DOMException(DOMException.WRONG_DOCUMENT_ERR, 
                                   DOMMessageFormatter.formatMessage(
                                   DOMMessageFormatter.XML_DOMAIN, 
                                   "WRONG_DOCUMENT_ERR", null));
        }
        LazyDocument doc = new LazyDocument(doctype, null);
        // If namespaceURI and qualifiedName are null return a Document with no document element.
        if (qualifiedName != null || namespaceURI != null) {
            Element e = doc.createElementNS(namespaceURI, qualifiedName);
            doc.appendChild(e);
        }
        return doc;
    }


    /**
     * Create a new LazyDocumentType
     * @see org.w3c.dom.DOMImplementation#createDocumentType
     */
    public DocumentType createDocumentType(String qualifiedName, 
                                           String publicID, 
                                           String systemID) {
        LazyDocumentType docType = new LazyDocumentType(null,
                                                        null,
                                                        qualifiedName,
                                                        publicID,
                                                        systemID,
                                                        null);
        return docType;
    }


    /**
     * Create a new LazyDocument from a template.
     * @see org.w3c.dom.DOMImplementation#createDocument
     */
    public Document createDocument(TemplateDOM templateDOM) throws DOMException {
        return new LazyDocument(null, templateDOM);
    }
}
