/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: LazyElement.java,v 1.3 2005-01-26 08:29:24 jkjome Exp $
 */

package org.enhydra.xml.lazydom;

import org.enhydra.xml.io.PreFormattedText;
import org.w3c.dom.Element;

/**
 * Interface implemented by both LazyElementNS and LazyElementNoNS.  These
 * must be split due to the underlying design to the Xerces DOM.
 * This interface provices a mechanism to access both.
 */
public interface LazyElement extends Element, LazyParent, PreFormattedText {
    /**
     * Mark the node as a template node and associated preformatted text.
     * @see LazyNode#makeTemplateNode
     * @see PreFormattedText#setPreFormattedText
     */
    public void makeTemplateNode(int nodeId,
                                 String text);

    /**
     * Get the template for this node.
     * @see LazyNode#getTemplateNode
     */
    public LazyElement getTemplateElement();

    /** 
     * Are the attributes of this node expanded?
     */
    public boolean areAttributesExpanded();

}
