/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: LazyParent.java,v 1.1.1.1 2003-03-10 16:36:18 taweili Exp $
 */

package org.enhydra.xml.lazydom;

import org.w3c.dom.Node;

/**
 * Interface use to define LazyDOM methods that apply to every node that
 * can contain other nodes.  See LazyNode documentation on properties
 * of node expansion.
 */
public interface LazyParent extends LazyNode {
    /**
     * Is the parent of this node expanded?
     */
    public boolean isParentExpanded();
    
    /**
     * Flag the parent as being expanded.
     */
    public void setParentExpanded();
    
    /**
     * Set the parent of this node during expansion.  This should also
     * flag the parent as being expanded.
     */
    //FIXME: Not used anymore???
    public void setParentWhileExpanding(Node parent);

    /**
     * Are the children of this node expanded?
     */
    public boolean areChildrenExpanded();

    /**
     * Flag the children as being expanded.
     */
    public void setChildrenExpanded();

    /**
     * Append a child during node expansion.  This should only add the
     * child, not trigger any other expansion.
     */
    public void appendChildWhileExpanding(Node child);
}
