/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: XMLCLogger.java,v 1.1.1.1 2003-03-10 16:36:19 taweili Exp $
 */

package org.enhydra.xml.xmlc;

/**
 * Abstract interface for XMLC logging.  Implementations of this class allows
 * the XMLC runtime to be integrate with different logging facilities.
 * Three levels of logging are currently used by the XMLC runtime:
 * <UL>
 * info logging - Log the occurance of normal events.
 * error logging - Log the occurance of errors.
 * debug logging - Log information useful for debugging the runtime problems.
 * </UL>
 */
public interface XMLCLogger {
    /**
     * Determine if info logging is enabled.
     */
    public boolean infoEnabled();

    /**
     * Login an info message.
     */
    public void logInfo(String msg);

    /**
     * Login an info message with exception.
     */
    public void logInfo(String msg,
                        Throwable except);

    /**
     * Determine if error logging is enabled.
     */
    public boolean errorEnabled();

    /**
     * Login an error message.
     */
    public void logError(String msg);

    /**
     * Login an error message with exception.
     */
    public void logError(String msg,
                         Throwable except);

    /**
     * Determine if debug logging is enabled.
     */
    public boolean debugEnabled();

    /**
     * Login an debug message.
     */
    public void logDebug(String msg);

    /**
     * Login an debug message with exception.
     */
    public void logDebug(String msg,
                         Throwable except);
}
