/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: JavaParameter.java,v 1.1.1.1 2003-03-10 16:36:19 taweili Exp $
 */

package org.enhydra.xml.xmlc.codegen;

/**
 * Class to store a method parameter.
 * Objects of this class are immutable.
 */
public final class JavaParameter extends JavaVariable {
    /**
     * Constructor with doc array.
     * @param name The parameter name.
     * @param type The parameter type.
     * @param doc The parameter documentation, where each row is a line of the
     *  document.  If null, no documention is created.
     * @see JavaModifiers
     */
    public JavaParameter(String name,
                         String type,
                         String[] doc) {
        super(name, type, 0, doc);
    }

    /**
     * Constructor with doc string.
     * @param name The parameter name.
     * @param type The parameter type.
     * @param doc The parameter documentation.
     *  If null, no documention is created.
     * @see JavaModifiers
     */
    public JavaParameter(String name,
                         String type,
                         String doc) {
        super(name, type, 0, doc);
    }

    /**
     * Print the param JavaDoc entry for this parameter, if there is any.
     */
    public void printJavaDoc(IndentWriter out) {
        String[] doc = getDoc();
        if (doc != null) {
            out.print(" * @param ");
            out.print(getName());
            out.print(' ');
            out.println(doc[0]);
            for (int idx = 1; idx < doc.length; idx++) {
                out.print(" * ");
                out.println(doc[idx]);
            }
        }
    }
}
