/*
 * Copyright (C) 2003, 2005, 2010 Richard Kunze, Jacob Kjome
 * 
 * This file is part of XMLC.
 *
 * XMLC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation, either version 2.1
 * of the License, or (at your option) any later version.
 *
 * XMLC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with XMLC.  If not,
 * see <http://www.gnu.org/licenses/>.
 */
package org.enhydra.xml.xmlc.deferredparsing;

import org.enhydra.xml.xmlc.XMLCRuntimeException;
import org.w3c.dom.Document;

/**
 * Interface for creating a DOM by document name. This interface is
 * used by XMLC generated classes with deferred parsing support to
 * delegate the actual work of DOM generation.  Note: Document loaders
 * may be shared between different XMLC classes and accessed
 * concurrently. Classes implementing this interface must be thread
 * safe (reentrant preferred).
 */
public interface DocumentLoader {

    /**
     * Bind to a factory.  This is done so that loggers, etc
     * can be grabbed from the factory.
     *
     * @param factory Factory that is creating the object.  Useful
     *  for getting logging object.
     */
    public void init(XMLCDeferredParsingFactory factory);

    /**
     * Provides a ResourceLoader implementation that abstracts resource paths
     * from the physical or contextual location of the paths.
     * 
     * @return a resource loader implementation
     * @since 2.3.2
     */
    public ResourceLoader getResourceLoader();

    /** 
     * Get a DOM instance representing the specified document.
     * @param docClass the XMLC generated class to get the DOM for.
     * @return the DOM representing objects of <code>docClass</code>
     */
    public Document getDocument(Class docClass) throws XMLCRuntimeException;
}
