package org.enhydra.xml.xmlc.deferredparsing;

import org.enhydra.xml.xmlc.dom.XMLCDomFactory;
import org.objectweb.asm.ClassWriter;
import org.objectweb.asm.FieldVisitor;
import org.objectweb.asm.Label;
import org.objectweb.asm.MethodVisitor;
import org.objectweb.asm.Opcodes;

class DynamicMLCreatorASM3Impl implements Opcodes, DynamicMLCreator {

    private String classNameDot;
    private String className;
    private String LclassName;
    private String LclassNameV;
    private String sourcePath;
    private String domFactoryNameDot;
    private String superClass;
    private String implInterfaces[];
    private String mimeType;
    private String encoding;

    public byte[] getByteArrayForClass() {
        return dump();
    }

    DynamicMLCreatorASM3Impl(String name, String path, XMLCDomFactory domFactory) {
        sourcePath = path;
        classNameDot = name;
        className = name.replace('.', '/');
        LclassName = "L" + className + ";";
        LclassNameV = "(" + LclassName + ")V";

        String[] factoryInterfaces = domFactory.getInterfaceNames();
        if (factoryInterfaces != null) {
            implInterfaces = new String[factoryInterfaces.length + 1];
        } else {
            implInterfaces = new String[1];
        }
        implInterfaces[0] = "org/enhydra/xml/xmlc/XMLObject";
        for (int i = 1; i < implInterfaces.length; i++) {
            implInterfaces[i] = factoryInterfaces[i-1].replace('.', '/');
        }

        domFactoryNameDot = domFactory.getClass().getName();
        mimeType = domFactory.getMIMEType();
        boolean isHtmlDocument = mimeType.indexOf("html") != -1;
        encoding = "UTF-8"; //real encoding defined at parse time since XMLC-2.3.2 (inputEncoding), so this just acts as a fall back default
        superClass = isHtmlDocument ? "org/enhydra/xml/xmlc/html/HTMLObjectImpl" : "org/enhydra/xml/xmlc/XMLObjectImpl";
    }

    private byte[] dump() {

        ClassWriter cw = new ClassWriter(ClassWriter.COMPUTE_MAXS);
        FieldVisitor fv;
        MethodVisitor mv;
        cw.visit(V1_1, ACC_PUBLIC + ACC_SUPER, className, null, superClass, implInterfaces);
        cw.visitSource("<generated>", null);
        {
            fv = cw.visitField(ACC_PUBLIC + ACC_FINAL + ACC_STATIC, "XMLC_GENERATED_CLASS", "Ljava/lang/Class;", null, null);
            fv.visitEnd();
        }

        {
            fv = cw.visitField(ACC_PUBLIC + ACC_FINAL + ACC_STATIC, "XMLC_SOURCE_FILE", "Ljava/lang/String;", null, sourcePath);
            fv.visitEnd();
        }

        {
            fv = cw.visitField(ACC_PRIVATE + ACC_FINAL + ACC_STATIC, "fDOMFactory", "Lorg/enhydra/xml/xmlc/dom/XMLCDomFactory;", null, null);
            fv.visitEnd();
        }

        {
            fv = cw.visitField(ACC_PRIVATE + ACC_FINAL, "fDocumentLoader", "Lorg/enhydra/xml/xmlc/deferredparsing/DocumentLoader;", null, null);
            fv.visitEnd();
        }

        {
            fv = cw.visitField(ACC_STATIC + ACC_SYNTHETIC, "class$0", "Ljava/lang/Class;", null, null);
            fv.visitEnd();
        }

        {
            fv = cw.visitField(ACC_STATIC + ACC_SYNTHETIC, "class$1", "Ljava/lang/Class;", null, null);
            fv.visitEnd();
        }

        {
            mv = cw.visitMethod(ACC_PUBLIC, "<init>", "()V", null, null);
            mv.visitCode();
            mv.visitVarInsn(ALOAD, 0);
            mv.visitMethodInsn(INVOKESTATIC, "org/enhydra/xml/xmlc/deferredparsing/StandardDocumentLoader", "getInstance", "()Lorg/enhydra/xml/xmlc/deferredparsing/DocumentLoader;");
            mv.visitMethodInsn(INVOKESPECIAL, className, "<init>", "(Lorg/enhydra/xml/xmlc/deferredparsing/DocumentLoader;)V");
            mv.visitVarInsn(ALOAD, 0);
            mv.visitMethodInsn(INVOKEVIRTUAL, className, "buildDocument", "()V");
            mv.visitInsn(RETURN);
            mv.visitMaxs(2, 1);
            mv.visitEnd();
        }

        {
            mv = cw.visitMethod(ACC_PUBLIC, "<init>", "(Z)V", null, null);
            mv.visitCode();
            mv.visitVarInsn(ALOAD, 0);
            mv.visitMethodInsn(INVOKESTATIC, "org/enhydra/xml/xmlc/deferredparsing/StandardDocumentLoader", "getInstance", "()Lorg/enhydra/xml/xmlc/deferredparsing/DocumentLoader;");
            mv.visitVarInsn(ILOAD, 1);
            mv.visitMethodInsn(INVOKESPECIAL, className, "<init>", "(Lorg/enhydra/xml/xmlc/deferredparsing/DocumentLoader;Z)V");
            mv.visitInsn(RETURN);
            mv.visitMaxs(3, 2);
            mv.visitEnd();
        }

        {
            mv = cw.visitMethod(ACC_PUBLIC, "<init>", LclassNameV, null, null);
            mv.visitCode();
            mv.visitVarInsn(ALOAD, 0);
            mv.visitMethodInsn(INVOKESPECIAL, superClass, "<init>", "()V");
            mv.visitVarInsn(ALOAD, 0);
            mv.visitVarInsn(ALOAD, 1);
            mv.visitMethodInsn(INVOKEVIRTUAL, className, "getDocumentLoader", "()Lorg/enhydra/xml/xmlc/deferredparsing/DocumentLoader;");
            mv.visitFieldInsn(PUTFIELD, className, "fDocumentLoader", "Lorg/enhydra/xml/xmlc/deferredparsing/DocumentLoader;");
            mv.visitVarInsn(ALOAD, 0);
            mv.visitVarInsn(ALOAD, 1);
            mv.visitMethodInsn(INVOKEVIRTUAL, className, "getDocument", "()Lorg/w3c/dom/Document;");
            mv.visitInsn(ICONST_1);
            mv.visitMethodInsn(INVOKEINTERFACE, "org/w3c/dom/Document", "cloneNode", "(Z)Lorg/w3c/dom/Node;");
            mv.visitTypeInsn(CHECKCAST, "org/w3c/dom/Document");
            mv.visitVarInsn(ALOAD, 1);
            mv.visitMethodInsn(INVOKEVIRTUAL, className, "getMIMEType", "()Ljava/lang/String;");
            mv.visitVarInsn(ALOAD, 1);
            mv.visitMethodInsn(INVOKEVIRTUAL, className, "getEncoding", "()Ljava/lang/String;");
            mv.visitMethodInsn(INVOKEVIRTUAL, className, "setDocument", "(Lorg/w3c/dom/Document;Ljava/lang/String;Ljava/lang/String;)V");
            mv.visitVarInsn(ALOAD, 0);
            mv.visitMethodInsn(INVOKEVIRTUAL, className, "syncAccessMethods", "()V");
            mv.visitInsn(RETURN);
            mv.visitMaxs(4, 2);
            mv.visitEnd();
        }

        {
            mv = cw.visitMethod(ACC_PUBLIC, "<init>", "(Lorg/enhydra/xml/xmlc/deferredparsing/DocumentLoader;Z)V", null, null);
            mv.visitCode();
            mv.visitVarInsn(ALOAD, 0);
            mv.visitMethodInsn(INVOKESPECIAL, superClass, "<init>", "()V");
            mv.visitVarInsn(ALOAD, 0);
            mv.visitVarInsn(ALOAD, 1);
            mv.visitFieldInsn(PUTFIELD, className, "fDocumentLoader", "Lorg/enhydra/xml/xmlc/deferredparsing/DocumentLoader;");
            mv.visitVarInsn(ILOAD, 2);

            Label l0 = new Label();
            mv.visitJumpInsn(IFEQ, l0);
            mv.visitVarInsn(ALOAD, 0);
            mv.visitMethodInsn(INVOKEVIRTUAL, className, "buildDocument", "()V");
            mv.visitLabel(l0);
            mv.visitInsn(RETURN);
            mv.visitMaxs(2, 3);
            mv.visitEnd();
        }

        {
            mv = cw.visitMethod(ACC_PUBLIC, "<init>", "(Lorg/enhydra/xml/xmlc/deferredparsing/DocumentLoader;)V", null, null);
            mv.visitCode();
            mv.visitVarInsn(ALOAD, 0);
            mv.visitVarInsn(ALOAD, 1);
            mv.visitInsn(ICONST_1);
            mv.visitMethodInsn(INVOKESPECIAL, className, "<init>", "(Lorg/enhydra/xml/xmlc/deferredparsing/DocumentLoader;Z)V");
            mv.visitInsn(RETURN);
            mv.visitMaxs(3, 2);
            mv.visitEnd();
        }

        {
            mv = cw.visitMethod(ACC_PUBLIC, "buildDocument", "()V", null, null);
            mv.visitCode();
            mv.visitVarInsn(ALOAD, 0);
            mv.visitVarInsn(ALOAD, 0);
            mv.visitMethodInsn(INVOKEVIRTUAL, className, "getDocumentLoader", "()Lorg/enhydra/xml/xmlc/deferredparsing/DocumentLoader;");
            mv.visitVarInsn(ALOAD, 0);
            mv.visitMethodInsn(INVOKEVIRTUAL, "java/lang/Object", "getClass", "()Ljava/lang/Class;");
            mv.visitMethodInsn(INVOKEINTERFACE, "org/enhydra/xml/xmlc/deferredparsing/DocumentLoader", "getDocument", "(Ljava/lang/Class;)Lorg/w3c/dom/Document;");
            mv.visitLdcInsn(mimeType);
            mv.visitLdcInsn(encoding);
            mv.visitMethodInsn(INVOKEVIRTUAL, className, "setDocument", "(Lorg/w3c/dom/Document;Ljava/lang/String;Ljava/lang/String;)V");
            mv.visitVarInsn(ALOAD, 0);
            mv.visitMethodInsn(INVOKEVIRTUAL, className, "syncAccessMethods", "()V");
            mv.visitInsn(RETURN);
            mv.visitMaxs(4, 1);
            mv.visitEnd();
        }

        {
            mv = cw.visitMethod(ACC_PUBLIC, "cloneNode", "(Z)Lorg/w3c/dom/Node;", null, null);
            mv.visitCode();
            mv.visitVarInsn(ALOAD, 0);
            mv.visitVarInsn(ILOAD, 1);
            mv.visitMethodInsn(INVOKEVIRTUAL, className, "cloneDeepCheck", "(Z)V");
            mv.visitTypeInsn(NEW, className);
            mv.visitInsn(DUP);
            mv.visitVarInsn(ALOAD, 0);
            mv.visitMethodInsn(INVOKESPECIAL, className, "<init>", LclassNameV);
            mv.visitInsn(ARETURN);
            mv.visitMaxs(3, 2);
            mv.visitEnd();
        }

        {
            mv = cw.visitMethod(ACC_PROTECTED + ACC_FINAL, "getDocumentLoader", "()Lorg/enhydra/xml/xmlc/deferredparsing/DocumentLoader;", null, null);
            mv.visitCode();
            mv.visitVarInsn(ALOAD, 0);
            mv.visitFieldInsn(GETFIELD, className, "fDocumentLoader", "Lorg/enhydra/xml/xmlc/deferredparsing/DocumentLoader;");
            mv.visitInsn(ARETURN);
            mv.visitMaxs(1, 1);
            mv.visitEnd();
        }

        {
            mv = cw.visitMethod(ACC_PROTECTED + ACC_FINAL, "getDomFactory", "()Lorg/enhydra/xml/xmlc/dom/XMLCDomFactory;", null, null);
            mv.visitCode();
            mv.visitFieldInsn(GETSTATIC, className, "fDOMFactory", "Lorg/enhydra/xml/xmlc/dom/XMLCDomFactory;");
            mv.visitInsn(ARETURN);
            mv.visitMaxs(1, 1);
            mv.visitEnd();
        }

        {
            mv = cw.visitMethod(ACC_PROTECTED, "syncWithDocument", "(Lorg/w3c/dom/Node;)V", null, null);
            mv.visitCode();
            mv.visitInsn(RETURN);
            mv.visitMaxs(0, 2);
            mv.visitEnd();
        }

        {
            mv = cw.visitMethod(ACC_STATIC + ACC_SYNTHETIC, "class$", "(Ljava/lang/String;)Ljava/lang/Class;", null, null);
            mv.visitCode();

            Label l0 = new Label();
            Label l1 = new Label();
            mv.visitTryCatchBlock(l0, l1, l1, "java/lang/ClassNotFoundException");
            mv.visitLabel(l0);
            mv.visitVarInsn(ALOAD, 0);
            mv.visitMethodInsn(INVOKESTATIC, "java/lang/Class", "forName", "(Ljava/lang/String;)Ljava/lang/Class;");
            mv.visitInsn(ARETURN);
            mv.visitLabel(l1);
            mv.visitVarInsn(ASTORE, 1);
            mv.visitTypeInsn(NEW, "java/lang/NoClassDefFoundError");
            mv.visitInsn(DUP);
            mv.visitVarInsn(ALOAD, 1);
            mv.visitMethodInsn(INVOKEVIRTUAL, "java/lang/Throwable", "getMessage", "()Ljava/lang/String;");
            mv.visitMethodInsn(INVOKESPECIAL, "java/lang/NoClassDefFoundError", "<init>", "(Ljava/lang/String;)V");
            mv.visitInsn(ATHROW);
            mv.visitMaxs(3, 2);
            mv.visitEnd();
        }

        {
            mv = cw.visitMethod(ACC_STATIC, "<clinit>", "()V", null, null);
            mv.visitCode();
            mv.visitFieldInsn(GETSTATIC, className, "class$0", "Ljava/lang/Class;");

            Label l0 = new Label();
            mv.visitJumpInsn(IFNONNULL, l0);
            mv.visitLdcInsn(classNameDot);
            mv.visitMethodInsn(INVOKESTATIC, className, "class$", "(Ljava/lang/String;)Ljava/lang/Class;");
            mv.visitInsn(DUP);
            mv.visitFieldInsn(PUTSTATIC, className, "class$0", "Ljava/lang/Class;");

            Label l1 = new Label();
            mv.visitJumpInsn(GOTO, l1);
            mv.visitLabel(l0);
            mv.visitFieldInsn(GETSTATIC, className, "class$0", "Ljava/lang/Class;");
            mv.visitLabel(l1);
            mv.visitFieldInsn(PUTSTATIC, className, "XMLC_GENERATED_CLASS", "Ljava/lang/Class;");
            mv.visitFieldInsn(GETSTATIC, className, "class$1", "Ljava/lang/Class;");

            Label l2 = new Label();
            mv.visitJumpInsn(IFNONNULL, l2);
            mv.visitLdcInsn(domFactoryNameDot);
            mv.visitMethodInsn(INVOKESTATIC, className, "class$", "(Ljava/lang/String;)Ljava/lang/Class;");
            mv.visitInsn(DUP);
            mv.visitFieldInsn(PUTSTATIC, className, "class$1", "Ljava/lang/Class;");

            Label l3 = new Label();
            mv.visitJumpInsn(GOTO, l3);
            mv.visitLabel(l2);
            mv.visitFieldInsn(GETSTATIC, className, "class$1", "Ljava/lang/Class;");
            mv.visitLabel(l3);
            mv.visitMethodInsn(INVOKESTATIC, "org/enhydra/xml/xmlc/dom/XMLCDomFactoryCache", "getFactory", "(Ljava/lang/Class;)Lorg/enhydra/xml/xmlc/dom/XMLCDomFactory;");
            mv.visitFieldInsn(PUTSTATIC, className, "fDOMFactory", "Lorg/enhydra/xml/xmlc/dom/XMLCDomFactory;");
            mv.visitInsn(RETURN);
            mv.visitMaxs(2, 0);
            mv.visitEnd();
        }

        cw.visitEnd();

        return cw.toByteArray();
    }
}
