/*
 * Copyright (C) 2010 Jacob Kjome
 * 
 * This file is part of XMLC.
 *
 * XMLC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation, either version 2.1
 * of the License, or (at your option) any later version.
 *
 * XMLC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with XMLC.  If not,
 * see <http://www.gnu.org/licenses/>.
 */
package org.enhydra.xml.xmlc.deferredparsing;

import java.net.URL;

/**
 * An interface providing for abstraction of resource loading by
 * {@link DocumentLoader document loader} implementations.
 * 
 * @author Jacob Kjome
 * @since 2.3.2
 */
public interface ResourceLoader {

    /**
     * Gets resources, such as XMLC templates and metadata files, from one of an
     * array of candidate paths relative to a DocumentLoader implementation
     * chosen context(s) in order of preference.  Paths are iterated until one is
     * found and its URL returned, or none are found and null is returned.
     * <p>
     * The candidatePaths may correspond to paths in XMLCDeferredParsingFactory
     * resource dirs, the classpath, a servlet context, etc...  It is entirely
     * up to implementors how and where to get resources corresponding to these
     * paths.
     * </p>
     * 
     * @param candidatePaths an array of candidate paths from which to load a
     *                      resource in order of preference
     * @return a URL corresponding to one of the candidatePaths or null if not
     *         found
     * @see DocumentLoader#getResourceLoader()
     */
    public URL getResource(String[] candidatePaths);

}
