/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: AccessorGenerator.java,v 1.2 2005-01-26 08:29:24 jkjome Exp $
 */

package org.enhydra.xml.xmlc.dom;

import org.enhydra.xml.xmlc.codegen.JavaClass;
import org.enhydra.xml.xmlc.codegen.JavaCode;
import org.enhydra.xml.xmlc.codegen.JavaMethod;
import org.enhydra.xml.xmlc.compiler.ElementInfo;

/**
 * Interface for class to generate access methods in a DOM-specific manner.
 */
public interface AccessorGenerator {
    /**
     * Create code to setup an access method for an element when building the
     * DOM.
     *
     * @param elementInfo Information about the element.
     * @param elementVarName The name of a variable that will contain
     *  the element during document construction at the point that
     *  the element is created.  The type of the variable will be Element.
     * @param body Append creation code to this code in the document builder
     *  that is being generated.
     */
    public void createAccessMethodInit(ElementInfo elementInfo,
                                       String elementVarName,
                                       JavaCode body);

    /**
     * Create code cached information so that element access methods
     * return null.
     *
     * @param elementInfo Information about the element.
     * @param body Append creation code to this code in the document builder
     *  that is being generated.
     */
    public void createNullElementAccess(ElementInfo elementInfo,
                                        JavaCode body);

    /**
     * Create code to reset the element access information from a node.
     *
     * @param elementInfo Information about the element.
     * @param nodeVarName The name of the variable that will contain
     *  element.  It will be of type Node, so it must be cast.
     * @param body Append creation code to this code in the document builder
     *  that is being generated.
     */
    public void createResetElementAccess(ElementInfo elementInfo,
                                         String nodeVarName,
                                         JavaCode body);

    /**
     * Create a getElementXXX() or getTagXXX() method for an element.
     *
     * @param elementInfo Information about the element.
     * @param accessor Infomation about the accessor to create.
     * @param createDelegation Should delegation support be created?
     * @param docClass The document class that is being generated.
     *  The method is added to this class.
     * @return The method that was created.  Used only in producing a listing.
     */
    public JavaMethod createAccessorMethod(ElementInfo elementInfo,
                                           ElementInfo.AccessorInfo accessor,
                                           boolean createDelegation,
                                           JavaClass docClass);
    
    /**
     * Create the setTextXXX() method for an element.
     *
     * @param elementInfo Information about the element.
     * @param createDelegation Should delegation support be created?
     * @param docClass The document class that is being generated.
     *  The method is added to this class.
     * @return The method that was created.  Used only in producing a listing.
     */
    public JavaMethod createSetTextMethod(ElementInfo elementInfo,
                                          boolean createDelegation,
                                          JavaClass docClass);
}
