/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: XercesDomFactory.java,v 1.7 2007-04-22 05:40:59 jkjome Exp $
 */

package org.enhydra.xml.xmlc.dom.xerces;

import org.apache.xerces.dom.DocumentImpl;
import org.apache.xerces.dom.DocumentTypeImpl;
import org.enhydra.xml.xmlc.XMLCError;
import org.enhydra.xml.xmlc.XMLObject;
import org.enhydra.xml.xmlc.XMLObjectLink;
import org.enhydra.xml.xmlc.dom.XMLCDomFactory;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentType;
import org.w3c.dom.Element;
import org.w3c.dom.Node;


/**
 * XMLC DOM factory for creating DocumentType and Document objects based on
 * the Xerces DOM.
 */
public class XercesDomFactory implements XMLCDomFactory {
    // FIXME: deprecated in 4.0 beta 2, can be removed when object code
    // compatibility XMLC 2.0.1 is no longer required.  This was removed
    // since generate a variable name to an inner class broke forte.
    /**
     * Class that adds XMLObjectLink support to the Xerces Document.
     * @deprecated Remains for compaitiblity with objects compiled
     *  with XMLC 2.0.1
     */
    public class LinkedXercesDocument extends DocumentImpl
                                      implements XMLObjectLink {
        /** Reference to XMLObject containing this Document. */
        private XMLObject fXmlObjectLink;

        /** Constructor. */
        public LinkedXercesDocument(DocumentType docType) {
            super(docType);
        }

        /** @see XMLObjectLink#setXMLObject */
        public void setXMLObject(XMLObject xmlObject) {
            fXmlObjectLink = xmlObject;
        }

        /** @see XMLObjectLink#getXMLObject */
        public XMLObject getXMLObject() {
            return fXmlObjectLink;
        }
    }

    /*
     * Suffix attached to interface name to form the implementation name.
     */
    private static final String IMPL_CLASS_SUFFIX = "Impl";

    /*
     * Xerces DOM to W3C package name map.
     */
    private static final String[][] DOM_PACKAGES_DOT_MAP = {
        {"org.apache.xerces.dom.", "org.w3c.dom."},
        {"org.apache.html.dom.", "org.w3c.dom.html."},
    };

    /*
     * Special-case element names to interface name that can't be
     * mapped just by changing the package.
     */
    private static final String[][] CLASS_INTERFACE_MAP = {
        {"org.apache.xerces.dom.ElementNSImpl", "org.w3c.dom.Element"},
        {"org.apache.xerces.dom.AttrNSImpl", "org.w3c.dom.Attr"},
    };

    /**
     * @return "text/xml"
     * @see XMLCDomFactory#getMIMEType
     */
    public String getMIMEType() {
        return "text/xml";
    }

    /**
     * @see XMLCDomFactory#getBaseClassName
     */
    public String getBaseClassName() {
        return "org.enhydra.xml.xmlc.XMLObjectImpl";
    }

    /**
     * @see XMLCDomFactory#getInterfaceNames
     */
    public String[] getInterfaceNames() {
        return null;
    }

    /**
     * @see XMLCDomFactory#getElementClassNames
     */
    public String[] getElementClassNames(Element element) {
        return null; // Not specified in XML.
    }

    /**
     * @see XMLCDomFactory#isURLAttribute
     */
    public boolean isURLAttribute(Element element,
                                  String attrName) {
        return false; // FIXME: Hook up to XLink??
    }

    /**
     * @see XMLCDomFactory#createDocumentType
     */
    public DocumentType createDocumentType(String qualifiedName,
                                           String publicId, 
                                           String systemId,
                                           String internalSubset) {
        DocumentTypeImpl docType = new DocumentTypeImpl(null, qualifiedName,
                                                        publicId, systemId);
        docType.setInternalSubset(internalSubset);
        return docType;
    }

    /**
     * This creates a Document object that extends the Xerces Document
     * class and implements XMLObjectLink.
     * @see XMLCDomFactory#createDocument
     */
    public Document createDocument(String namespaceURI,
                                   String qualifiedName,
                                   DocumentType docType) {
        Document doc = new XercesLinkedDocument(docType);
        // Create the document element
        doc.appendChild(doc.createElementNS(namespaceURI, qualifiedName));
        return doc;
    }

    /**
     * @see XMLCDomFactory#getDocumentClassName
     */
    public String getDocumentClassName() {
        return XercesLinkedDocument.class.getName();
    }

    /**
     * @see XMLCDomFactory#nodeClassToInterface
     */
    public String nodeClassToInterface(Node node) {
        String className = node.getClass().getName();

        // Search explict class name mappings.
        for (int idx = 0; idx < CLASS_INTERFACE_MAP.length; idx++) {
            if (className.equals(CLASS_INTERFACE_MAP[idx][0])) {
                return CLASS_INTERFACE_MAP[idx][1];
            }
        }

        // Search for package name mapping
        int packageIdx;
        for (packageIdx = 0; packageIdx < DOM_PACKAGES_DOT_MAP.length; packageIdx++) {
            if (className.startsWith(DOM_PACKAGES_DOT_MAP[packageIdx][0])) {

                break;
            }
        }
        if (packageIdx == DOM_PACKAGES_DOT_MAP.length) {
            throw new XMLCError("Can't determine DOM interface for node of class \""
                                + className
                                + "\" (maybe be mismatch between XMLCDomFactory DOM implementation)");
        }

        // Remove the suffix to get the base class name.
        int suffixIdx = className.lastIndexOf(IMPL_CLASS_SUFFIX);
        if (suffixIdx < 0) {
            throw new XMLCError("Class \"" + className
                                + "\" does not have suffix \"" + IMPL_CLASS_SUFFIX
                                + "\" (maybe be mismatch between XMLCDomFactory DOM implementation)");
        }
        
        return DOM_PACKAGES_DOT_MAP[packageIdx][1]
            + className.substring(DOM_PACKAGES_DOT_MAP[packageIdx][0].length(),
                                  suffixIdx);
    }

}
