/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: HTMLDocumentFactory.java,v 1.2 2005-01-26 08:29:24 jkjome Exp $
 */

package org.enhydra.xml.xmlc.html;

import org.enhydra.xml.xmlc.dom.XMLCDomFactory;
import org.enhydra.xml.xmlc.dom.xerces.XercesHTMLDomFactory;
import org.w3c.dom.html.HTMLDocument;

/**
 * Factory class for constructing new HTMLDocument objects without
 * needing to know the implementing class.
 */
public class HTMLDocumentFactory {
    /**
     * Factory for creating documents.
     */
    private static XMLCDomFactory domFactory = new XercesHTMLDomFactory();

    /**
     * Constructor that prevents instantiation.
     */
    private HTMLDocumentFactory() {
    }

    /**
     * Create a new HTMLDocument object.  The document will contain an
     * empty HTML element.
     */
    static public HTMLDocument createDocument() {
        return (HTMLDocument)domFactory.createDocument(null, null, null);
    }

    /**
     * Create a new HTMLDocument object, initialized with
     * empty HTML, HEAD, TITLE and BODY elements.
     * @param title The title string to set in the header.
     */
    static public HTMLDocument createBasicDocument(String title) {
        HTMLDocument doc = createDocument();

        // This creates head and title
        doc.setTitle(title);
        // This creates body
        doc.getBody();

        return doc;
    }

    /**
     * Create a new HTMLDocument object, initialized with
     * empty HTML, HEAD, TITLE and BODY elements.
     */
    static public HTMLDocument createBasicDocument() {
        return createBasicDocument("");
    }
}
