/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: DOMEdits.java,v 1.2 2005-01-26 08:29:24 jkjome Exp $
 */

package org.enhydra.xml.xmlc.metadata;

import org.w3c.dom.Document;

/**
 * Section containing DOM editing specifications.  These are modifications
 * done to elements in the DOM during the compilation of a document.
 */
public class DOMEdits extends MetaDataElement {
    /**
     * Element name.
     */
    public static final String TAG_NAME = "domEdits";

    /**
     * Constructor.
     */
    public DOMEdits(Document ownerDoc) {
    	super(ownerDoc, TAG_NAME);
    }

    /**
     * Get the URLMapping child elements.
     */
    public URLMapping[] getURLMappings() {
        return (URLMapping[])getChildren(URLMapping.class);
    }

    /**
     * Add a URLMapping child element.
     */
    public void addURLMapping(URLMapping urlMapping) {
        appendChild(urlMapping);
    }

    /**
     * Create and add a URLMapping child element.
     */
    public URLMapping addURLMapping() {
        URLMapping urlMapping = new URLMapping(getOwnerDocument());
        appendChild(urlMapping);
        return urlMapping;
    }

    /**
     * Delete a URLMapping child element.
     */
    public void deleteURLMapping(URLMapping urlMapping) {
        removeChild(urlMapping);
    }

    /**
     * Get the URLRegExpMapping child elements.
     */
    public URLRegExpMapping[] getURLRegExpMappings() {
        return (URLRegExpMapping[])getChildren(URLRegExpMapping.class);
    }

    /**
     * Add a URLRegExpMapping child element.
     */
    public void addURLRegExpMapping(URLRegExpMapping urlRegExpMapping) {
        appendChild(urlRegExpMapping);
    }

    /**
     * Create and add a URLRegExpMapping child element.
     */
    public URLRegExpMapping addURLRegExpMapping() {
        URLRegExpMapping urlRegExpMapping
            = new URLRegExpMapping(getOwnerDocument());
        appendChild(urlRegExpMapping);
        return urlRegExpMapping;
    }

    /**
     * Delete a URLRegExpMapping child element.
     */
    public void deleteURLRegExpMapping(URLRegExpMapping urlRegExpMapping) {
        removeChild(urlRegExpMapping);
    }

    /**
     * Get the DeleteElement child elements.
     */
    public DeleteElement[] getDeleteElements() {
        return (DeleteElement[])getChildren(DeleteElement.class);
    }

    /**
     * Add a DeleteElement child element.
     */
    public void addDeleteElement(DeleteElement deleteElement) {
        appendChild(deleteElement);
    }

    /**
     * Create and add a DeleteElement child element.
     */
    public DeleteElement addDeleteElement() {
        DeleteElement deleteElement = new DeleteElement(getOwnerDocument());
        appendChild(deleteElement);
        return deleteElement;
    }

    /**
     * Delete a DeleteElement child element.
     */
    public void deleteDeleteElement(DeleteElement deleteElement) {
        removeChild(deleteElement);
    }
}
