/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: HTMLTag.java,v 1.1.1.1 2003-03-10 16:36:23 taweili Exp $
 */

package org.enhydra.xml.xmlc.metadata;

import org.w3c.dom.Document;

/**
 * Add a proprietary tag to set of valid html tags.
 */
public class HTMLTag extends MetaDataElement implements HTMLTagDef {
    /**
     * Element name.
     */
    public static final String TAG_NAME = "htmlTag";

    /**
     * Attribute names.
     */
    private static final String NAME_ATTR = "name";
    private static final String INLINE_ATTR = "inline";
    private static final String BLOCK_ATTR = "block";
    private static final String EMPTY_ATTR = "empty";
    private static final String OPT_CLOSE_ATTR = "optclose";

    /**
     * Constructor.
     */
    public HTMLTag(Document ownerDoc) {
    	super(ownerDoc, TAG_NAME);
    }

    /**
     * Get the name attribute value.
     */
    public String getName() {
        return getAttributeNull(NAME_ATTR);
    }

    /**
     * Set the name attribute value.
     */
    public void setName(String value) {
        setRemoveAttribute(NAME_ATTR, value);
    }

    /**
     * Get the inline attribute value.
     */
    public boolean getInline() {
        return getBooleanAttribute(INLINE_ATTR);
    }

    /**
     * Set the inline attribute value.
     */
    public void setInline(boolean value) {
        setBooleanAttribute(INLINE_ATTR, value);
    }

    /**
     * Get the block attribute value.
     */
    public boolean getBlock() {
        return getBooleanAttribute(BLOCK_ATTR);
    }

    /**
     * Set the block attribute value.
     */
    public void setBlock(boolean value) {
        setBooleanAttribute(BLOCK_ATTR, value);
    }

    /**
     * Get the empty attribute value.
     */
    public boolean getEmpty() {
        return getBooleanAttribute(EMPTY_ATTR);
    }

    /**
     * Set the empty attribute value.
     */
    public void setEmpty(boolean value) {
        setBooleanAttribute(EMPTY_ATTR, value);
    }

    /**
     * Get the optclose attribute value.
     */
    public boolean getOptclose() {
        return getBooleanAttribute(OPT_CLOSE_ATTR);
    }

    /**
     * Set the optclose attribute value.
     */
    public void setOptclose(boolean value) {
        setBooleanAttribute(OPT_CLOSE_ATTR, value);
    }
}

