/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: Include.java,v 1.2 2005-01-26 08:29:24 jkjome Exp $
 */

package org.enhydra.xml.xmlc.metadata;

import org.w3c.dom.Document;

/**
 * Specifies an included file for dependencies.
 */
public class Include extends MetaDataElement {
    /**
     * Element name.
     */
    public static final String TAG_NAME = "include";

    /**
     * Attribute names.
     */
    private static final String URL_ATTR = "url";

    /**
     * Constructor.
     */
    public Include(Document ownerDoc) {
    	super(ownerDoc, TAG_NAME);
    }

    /**
     * Constructor for derived class.
     */
    public Include(Document ownerDoc,
                   String tagName) {
    	super(ownerDoc, tagName);
    }

    /**
     * Get the url attribute.
     */
    public String getUrl() {
        return getAttributeNull(URL_ATTR);
    }

    /**
     * Set the url attribute.
     */
    public void setUrl(String value) {
        setRemoveAttribute(URL_ATTR, value);
    }

    /**
     * Get the include direct children.
     */
    public Include[] getIncludes() {
        return (Include[])getChildren(Include.class);
    }

    /**
     * Add a include child
     */
    public void addInclude(Include include) {
        appendChild(include);
    }
}
