/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: ParserType.java,v 1.2 2006-08-10 23:57:26 jkjome Exp $
 */

package org.enhydra.xml.xmlc.metadata;

/**
 * Enumerated type for the parser XMLC will use.
 */
public class ParserType extends EnumeratedType {
    /**
     * Constant for Xerces XML parser.
     */
    public static final ParserType XERCES = new ParserType("xerces");

    /**
     * Constant for Tidy HTML parser.
     */
    public static final ParserType TIDY = new ParserType("tidy");

    /**
     * Constant for Swing HTML parser.
     */
    public static final ParserType SWING = new ParserType("swing");

    /**
     * Constant for NekoHTML parser.
     */
    public static final ParserType NEKOHTML = new ParserType("nekohtml");

    /**
     * Internal constructor.  Only a fixed set of types are allowed.
     */
    private ParserType(String name) {
        super(name);
    }

    /**
     * Get the object for the specified type.  If desiredType is null,
     * return null.
     */
    public static ParserType getType(String desiredType) {
        if (desiredType == null) {
            return null;
        } else if (XERCES.fName.equals(desiredType)) {
            return XERCES;
        } else if (TIDY.fName.equals(desiredType)) {
            return TIDY;
        } else if (SWING.fName.equals(desiredType)) {
            return SWING;
        } else if (NEKOHTML.fName.equals(desiredType)) {
            return NEKOHTML;
        } else {
            throw new IllegalArgumentException("Invalid ParserType: \""
                                               + desiredType
                                               + "\", expected of \""
                                               + XERCES.fName + "\", \""
                                               + TIDY.fName + "\", \""
                                               + SWING.fName + "\", or \""
                                               + NEKOHTML.fName + "\"");
        }
    }
}
