/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: URLMapping.java,v 1.2 2005-01-26 08:29:24 jkjome Exp $
 */

package org.enhydra.xml.xmlc.metadata;

import org.w3c.dom.Document;

/**
 * Specifies the literal replacement of URLs in element attributes. When used
 * in the &lt;domEdits&gt; section, it applies globally to elements.  When use
 * in a element or sub-document, it applies only to that context.
 */
public class URLMapping extends URLEdit {
    /**
     * Element name.
     */
    public static final String TAG_NAME = "urlMapping";

    /**
     * Attribute names.
     */
    private static final String URL_ATTR = "url";
    private static final String NEW_URL_ATTR = "newUrl";

    /**
     * Constructor.
     */
    public URLMapping(Document ownerDoc) {
    	super(ownerDoc, TAG_NAME);
    }

    /**
     * Get the url attribute value.
     */
    public String getUrl() {
        return getAttributeNull(URL_ATTR);
    }

    /**
     * Set the url attribute value.
     */
    public void setUrl(String value) {
        setRemoveAttribute(URL_ATTR, value);
    }

    /**
     * Get the newUrl attribute value.
     */
    public String getNewUrl() {
        return getAttributeNull(NEW_URL_ATTR);
    }

    /**
     * Set the newUrl attribute value.
     */
    public void setNewUrl(String value) {
        setRemoveAttribute(NEW_URL_ATTR, value);
    }
}
