/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: SimpleDiffFilter.java,v 1.3 2006-03-04 05:56:07 jkjome Exp $
 */

package org.enhydra.xml.driver;


/**
 * Filter for simple string matches in input
 */
public class SimpleDiffFilter implements TestDiff.LineFilter {
    /** Pattern and replacement */
    private String fPattern;
    private String fReplace;
    private char fPathNormalizerOldChar;
    private char fPathNormalizerNewChar;
    
    /** 
     * Constructor for standard test case name
     */
    public SimpleDiffFilter(String pattern,
                            String replace) {
        fPattern = pattern;
        fReplace = replace;
    }

    /**
     * Optional. Allows for post-pattern replacement path normalization.
     * This may be important if paths in expected files are defined as
     * Unix paths while the code calling filter(String) is running
     * on Windows.  Or, if a "." package paths are wanted to be replaced
     * by "/" package paths.  Normalizatoin skipped unless both parameters
     * are set to something other than the char default value '\u0000'.
     *
     * @see String#replace(char, char)
     */
    public SimpleDiffFilter(String pattern,
                            String replace,
                            char pathNormalizerOldChar,
                            char pathNormalizerNewChar) {
        this(pattern, replace);
        fPathNormalizerOldChar = pathNormalizerOldChar;
        fPathNormalizerNewChar = pathNormalizerNewChar;
    }

    /** Filter a line */
    public String filter(String line) {
        int nextIdx = line.indexOf(fPattern);
        if (nextIdx < 0) {
            return line;  // fast-path, no match
        }

        // Match, substitute all instances
        StringBuffer buf = new StringBuffer(line.length());
        int startIdx = 0;
        int len = fPattern.length();
        while (nextIdx >= 0) {
            buf.append(line.substring(startIdx, nextIdx));
            buf.append(fReplace);
            startIdx = nextIdx + len;
            nextIdx = line.indexOf(fPattern, startIdx);
        }
        if (startIdx < line.length()) {
            buf.append(line.substring(startIdx));
        }

        String result = buf.toString();
        if (fPathNormalizerOldChar != '\u0000' && fPathNormalizerNewChar != '\u0000') {
            result = result.replace(fPathNormalizerOldChar, fPathNormalizerNewChar);
        }
        return result;
    }
}
