/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: OutputDocument.java,v 1.3 2005-01-26 08:29:24 jkjome Exp $
 */

package org.enhydra.xml.xmlc.driver;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;

import org.enhydra.xml.dom.DOMInfo;
import org.enhydra.xml.dom.DOMStats;
import org.enhydra.xml.driver.TestException;
import org.enhydra.xml.driver.TestFileOps;
import org.enhydra.xml.io.DOMFormatter;
import org.enhydra.xml.io.OutputOptions;
import org.enhydra.xml.xmlc.XMLObject;
import org.w3c.dom.Document;

/**
 * Output or dump information about a compiled document.
 */
public class OutputDocument {
    /** Default formatter to use when no output options are specified */
    private static DOMFormatter fDefaultFormatter = new DOMFormatter();

    /** Write with toDocuemnt() */
    public static void writeWithToDoc(XMLObject xmlObj,
                                      File outFile) {
        try {
            PrintWriter out = new PrintWriter(new FileOutputStream(outFile));
            try {
                out.println(xmlObj.toDocument());
            } finally {
                out.close();
            }
        } catch (Throwable err) {
            throw new TestException(err);
        }
    }
    
    /*
     * Write a document using a DOMFormatter
     */
    public static void write(XMLObject xmlObj,
                             File outFile,
                             OutputOptions options) {
        DOMFormatter formatter;
        if (options == null) {
            formatter = fDefaultFormatter;
        } else {
            formatter = new DOMFormatter(options);
        }

        try {
            OutputStream out = new FileOutputStream(outFile);
            try {
                formatter.write(xmlObj, out);
            } finally {
                out.close();
            }
        } catch (Throwable err) {
            throw new TestException(err);
        }

    }

    /** Do DOM dump */
    private static void doDump(Document doc,
                              File outFile) throws IOException {
        TestFileOps.ensureFileDir(outFile);

        // Force consistent encoding
        PrintWriter out
            = new PrintWriter(new OutputStreamWriter(
                                new FileOutputStream(outFile), "UTF-8"));
        try {
            if (doc instanceof XMLObject) {
                XMLObject xmlObj = (XMLObject)doc;
                out.println("Document MIME type: " + xmlObj.getMIMEType());
                out.println("Document encoding: " + (xmlObj.getInputEncoding() != null ? xmlObj.getInputEncoding() : xmlObj.getXmlEncoding()));
            }
            DOMInfo.printTree("DOM hierarchy", doc,
                              DOMInfo.PRINT_ATTR_DETAILS, out);
            DOMStats.printStats("DOM statistics", doc,
                                DOMStats.SIMPLE_CLASS_NAMES, out);
        } finally {
            out.close();
        }
    }

    /**
     * Dump DOM and other docuement information to a file.
     * Additional information is provided if the document is a
     * XMLObject.
     */
    public static void dump(Document doc,
                            File outFile) {
        try {
            doDump(doc, outFile);
        } catch (Throwable err) {
            throw new TestException(err);
        }
    }
}
