/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: TestDOMSource.java,v 1.5 2006-09-04 05:29:41 jkjome Exp $
 */

package org.enhydra.xml.xmlc.driver;
import java.io.File;

import org.enhydra.xml.driver.TestCaseBase;
import org.enhydra.xml.xmlc.XMLCFactory;
import org.enhydra.xml.xmlc.XMLObject;
import org.enhydra.xml.xmlc.deferredparsing.XMLCDeferredParsingFactory;

//FIXME: turn into cache of compiled objects based on attributes. and file

/**
 * Object used to create instances of test documents that are used
 * in API testing.  This will compile a document once for a set of
 * test parameters.  Other requests for instances of the document are
 * created from the same class.  An instance of this object is created per
 * document and set of parameters and is often shared by multiple tests.
 */
public class TestDOMSource {
    /** Test case we are assocaited with. */
    private TestCaseBase fTest;

    /** Object containing test parameters */
    private XmlcTestParams fParams;

    /** Information about the document */
    File fInputFile;
    String fUnqualClassName;
    Class fImpls;

    /** 
     * Factory use to create instances of the class.  Created on first use;
     * class will only be compiled onces per test case execution.  Also the
     * class name of the document class.
     */
    private XMLCFactory fFactory;
    private String fClassName;

    /** 
     * Constructor.
     * @params params may differ from test parms.
     * @param impls Interface to implement, maybe null.
     */
    public TestDOMSource(TestCaseBase test,
                         XmlcTestParams params,
                         File inputFile,
                         String unqualClassName,
                         Class impls) {
        fTest = test;
        fParams = params;
        fInputFile = inputFile;
        fUnqualClassName = unqualClassName;
        fClassName = fTest.getTestPackage() + "." + unqualClassName;
        fImpls = impls;
    }

    /** Get the input file name */
    public File getInputFile() {
        return fInputFile;
    }

    /*
     * Compile the source file and setup the factory to create instances of
     * it.
     */
    private void compile() {
        File outFile = fTest.getResultFile(fUnqualClassName,
                                           XmlcTestCaseBase.COMPILE_OUTPUT_EXT);
        File expectFile = fTest.getExpectedFile(fUnqualClassName,
                                                XmlcTestCaseBase.COMPILE_OUTPUT_EXT);
        
        ExecXmlc execXmlc = new ExecXmlc(fTest, fParams);
        execXmlc.addOpt(ExecXmlc.OPT_CLASS, fClassName);
        if (fImpls != null) {
            execXmlc.addOpt(ExecXmlc.OPT_IMPLEMENTS,
                            fImpls.getName());
        }
        execXmlc.addOpt(ExecXmlc.OPT_DEST_DIR, fTest.getClassRoot());
        execXmlc.addClassPath(fTest.getClassRoot());
        execXmlc.setSrcFile(fInputFile);
        execXmlc.compile(outFile);
        fTest.getDiffer().diff(expectFile, outFile);
    }

    /**
     * Get an instance of the document object, compile if needed.
     */
    public synchronized XMLObject create() {
        if (fFactory == null) {
            compile();

	    //if (fParams.getIsDeferredParsing()) {
		java.io.PrintWriter writer 
		    = new java.io.PrintWriter(new java.io.OutputStreamWriter(System.err));
		org.enhydra.xml.xmlc.StreamXMLCLogger logger 
		    = new org.enhydra.xml.xmlc.StreamXMLCLogger(writer, writer, writer);

		XMLCDeferredParsingFactory dpf = new XMLCDeferredParsingFactory (null, fTest.createClassLoader(), logger); // use reloading loader
		dpf.addResourceDir(fTest.getInputRoot().getAbsolutePath());
		dpf.addPackagePrefix(fTest.getTestPackage());
		fFactory = dpf;
	    //} else {
		//fFactory = new XMLCStdFactory(fTest.createClassLoader(), null); // use standard loader
	    //}
	}
	return fFactory.create(fClassName);
    }
}
