/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: EncodingTests.java,v 1.3 2006-08-31 04:07:59 jkjome Exp $
 */

package org.enhydra.xml.xmlc.html;
import java.lang.reflect.Method;

import junit.framework.Test;

import org.enhydra.xml.xmlc.driver.ExecXmlc;

/**
 * Tests of character set and encoding.
 */
public class EncodingTests extends HtmlTestCaseBase {

    /** Factory method to create suite of these tests */
    public static Test suite() {
        return createSuite(EncodingTests.class, null);
    }
    
    /** Constructor */
    public EncodingTests(Method method) {
        super(method);
    }

    /** Create a new test object with encoding */
    private HtmlBasicTest createTest(String encoding,
                                     String inputDoc) {
        HtmlBasicTest test = createTest(inputDoc);
        test.getOutputOptions().setEncoding(encoding);
        test.addBoth(ExecXmlc.OPT_HTML_ENCODING, encoding);
        return test;
    }

    /*
     * Test 1: Chinese big-5 (This page has illegal HTML and tidy
     * discards several bogus constructs).
     */
    public void test1() {
        HtmlBasicTest test = createTest("BIG5",
                                        "encodings/newscontent_big5.html");
        test.basicTest();
    }

    /*
     * Test 2: Chinese big-5
     */
    public void test2() {
        HtmlBasicTest test = createTest("BIG5", 
                                        "encodings/big5.page1.html");
        test.basicTest();
    }

    /*
     * Test 3: Chinese GB
     */
    public void test3() {
        HtmlBasicTest test = createTest("GB2312",
                                        "encodings/gb.page1.html");
        test.basicTest();
    }

    /*
     * Test 4: Japanese
     */
    public void test4() {
        HtmlBasicTest test = createTest("ISO-2022-JP",
                                        "encodings/iso-2022-jp.page1.html");
        
        // FIXME: reversing the test in the page generates an invalid
        // character that generates an error when java reads it.  Not sure
        // why, as should be ok. Just disable for now, as it's a test issue
        // not an issue with the code being tests.
        test.setEditReloadSource(false);

        test.basicTest();
    }

    /*
     * Test 5:  Windows Thai MS874
     */
    public void test5() {
        // (FIXME: Don't have a HTML file for this)
    }

    /*
     * Test 6: Chinese big-5 with -ssi
     */
    public void test6() {
        HtmlBasicTest test = createTest("BIG5",
                                        "encodings/newscontent_ssi_big5.html");
        test.addFirst(ExecXmlc.OPT_SSI);
        test.addExtraSrcFile(getInputFile("encodings/newscontent_ssi_incl_big5.html"));
        test.basicTest();
    }

    /*
     * Test 7: Emoji characters
     */
    public void test7() {
        HtmlBasicTest test = createTest("Shift_JIS",
                                        "encodings/emoji.chtml");
        test.basicTest();
    }
}
