/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: ErrorHandlingTests.java,v 1.2 2005-01-26 08:29:25 jkjome Exp $
 */

package org.enhydra.xml.xmlc.html;
import java.io.File;
import java.lang.reflect.Method;

import junit.framework.Test;

import org.enhydra.xml.xmlc.driver.ExecXmlc;

/**
 * Tests of error handling.  Several problems are only generate errors with
 * swing.
f */
public class ErrorHandlingTests extends HtmlTestCaseBase {

    /** Factory method to create suite of these tests */
    public static Test suite() {
        return createSuite(ErrorHandlingTests.class, null);
    }
    
    /** Constructor */
    public ErrorHandlingTests(Method method) {
        super(method);
    }

    /*
     * Test 1: Java script not in comment (will not work with swing parser)
     */
    public void test1() {
        HtmlBasicTest test = createTest("javascript1.html");
        // Using swing parser should give an error.
        if (fParams.getParser().equals(ExecXmlc.SWING_PARSER)) {
            test.setCompileShouldFail();
        }
        test.basicTest();
    }

    /*
     * Test 2: 
     */
    public void test2() {
        HtmlBasicTest test = createTest("bogus1.html");
        // Using swing parser should give an error.
        if (fParams.getParser().equals(ExecXmlc.SWING_PARSER)) {
            test.setCompileShouldFail();
        }
        test.basicTest();
    }

    /*
     * Test 3:
     */
    public void test3() {
        HtmlBasicTest test = createTest("bogus2.html");
        // Using swing parser should give an error.
        if (fParams.getParser().equals(ExecXmlc.SWING_PARSER)) {
            test.setCompileShouldFail();
        }
        test.addCyberStudioTags();
        test.basicTest();
    }

    /*
     * Test 4: Attempt to parse a XML file
     */
    public void test4() {
        File insulin3 = getOtherInputFile("../xml", "bioml/insulin3.bioml");
        HtmlBasicTest test = createTest(insulin3);
        test.setCompileShouldFail();
        test.basicTest();
    }

    /*
     * Test 5:  Bogus use of XML empty content tag in HTML
     * (swing doesn't report error, Tidy was hanging).  
     */
    public void disable_test5() {
        // FIXME: Tidy isn't reporting an error either, just disable this
        // test 
        HtmlBasicTest test = createTest("bogusTag.html");
        // not detected by swing
        if (!fParams.getParser().equals(ExecXmlc.SWING_PARSER)) {
            test.setCompileShouldFail();
        }
        test.basicTest();
    }
}
