/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: SSITests.java,v 1.5 2006-08-31 04:51:06 jkjome Exp $
 */

package org.enhydra.xml.xmlc.html;
import java.io.File;
import java.lang.reflect.Method;

import junit.framework.Test;

import org.enhydra.xml.driver.TestFileOps;
import org.enhydra.xml.xmlc.driver.ExecXmlc;

/**
 * Tests of SSI functionality.
 */
public class SSITests extends HtmlTestCaseBase {
    /** Factory method to create suite of these tests */
    public static Test suite() {
        return createSuite(SSITests.class, null);
    }

    /** Constructor */
    public SSITests(Method method) {
        super(method);
    }

    /*
     * Test 1:  Parse file without processing SSI
     */
    public void test1() {
        HtmlBasicTest test = createTest("ssiTestPage1.html");
        test.addOptionFile(getInputFile("testPage1.xmlc"));
        test.basicTest();
    }

    /*
     * Test 2: Now process SSI.
     */
    public void test2() {
        HtmlBasicTest test = createTest("ssiTestPage1.html");
        //  Need to copy included file if we are doing a recomp tests
        //if (fParams.getReloading() || fParams.getIsDeferredParsing()) {
            TestFileOps.copyFileToDir(getInputFile("ssiTestIncl1.html"),
                                      getResultsDir());
        //}
        test.addFirst(ExecXmlc.OPT_SSI);
        test.addOptionFile(getInputFile("testPage1.xmlc"));
        test.basicTest();
    }

    /*
     * Test 3: Include with relative path
     */
    public void test3() {
        HtmlBasicTest test = createTest("ssiTestPage2.html");
        test.addFirst(ExecXmlc.OPT_SSI);
        test.addOptionFile(getInputFile("testPage1.xmlc"));
        test.basicTest();
    }

    /*
     * Test 4: This test file broke SSI when it had CR/LF EOLNs.
     * This was not actually a problem with CR/LF, but with SSI's buffer
     * expansion.
     */
    public void test4() {
        HtmlBasicTest test = createTest("cr-lf/Login.html");
        //if (fParams.getReloading() || fParams.getIsDeferredParsing()) {
            TestFileOps.copyFileToDir(getInputFile("cr-lf/TopBanner.ssi"),
                    new File(getPackageDir(), "../cr-lf"));
            TestFileOps.copyFileToDir(getInputFile("cr-lf/Footer.ssi"),
                    new File(getPackageDir(), "../cr-lf"));
        //}
        test.addFirst(ExecXmlc.OPT_SSI);
        test.basicTest();
    }
}
