/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: BasicTests.java,v 1.3 2007-11-24 21:45:40 jkjome Exp $
 */

package org.enhydra.xml.xmlc.metadata;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.lang.reflect.Method;

import junit.framework.Test;

import org.enhydra.xml.dom.DOMInfo;
import org.enhydra.xml.driver.TestCaseBase;
import org.enhydra.xml.driver.TestException;
import org.enhydra.xml.io.ErrorReporter;
import org.enhydra.xml.xmlc.XMLCException;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

/**
 * Tests of metadata file parsing.  
 */
public class BasicTests extends TestCaseBase {
    /** XMLC schema */
    //private static final String XMLC_XSD
    //    = "../../../../../../org/enhydra/xml/xmlc/metadata/xmlc-1.0.2.xsd";

    /** Factory method to create suite of these tests */
    public static Test suite() {
        return createSuite(BasicTests.class, null);
    }
    
    /** Constructor */
    public BasicTests(Method method) {
        super(method);
    }

    /**
     * Parse file, dump and validate.
     */
    private MetaDataDocument parseDump(File xmlcFile,
                                       String outExt) 
        throws IOException,
               SAXException,
               XMLCException {
        File outFile = getResultFile(outExt);
        File expectFile = getExpectedFile(outExt);

        outFile.getParentFile().mkdirs();
        PrintWriter out =  new PrintWriter(new FileWriter(outFile));
        MetaDataDocument metaDataDoc;
        try {
            metaDataDoc
                = MetaDataDocument.parseMetaData(new InputSource(xmlcFile.getPath()),
                                                 new ErrorReporter(out),
                                                 null);
            DOMInfo.printTree(xmlcFile.getPath(), metaDataDoc, out);
        } finally {
            out.close();
        }
        getDiffer().diff(expectFile, outFile);
        return metaDataDoc;
    }

        
    /**
     * Common routine used to implement the tests
     */
    private void testParseMetaData(File xmlcFile) {
        try {
            MetaDataDocument doc = parseDump(xmlcFile, "1.dump");
            File docOut = getResultFile("xmlc");
            doc.serialize(docOut);
            parseDump(docOut, "2.dump");
        } catch (IOException except) {
            throw new TestException(except);
        } catch (SAXException except) {
            throw new TestException(except);
        } catch (XMLCException except) {
            throw new TestException(except);
        }
    }

    /**
     * Test 1: Parse file with most tags
     */
    public void test1() {
        testParseMetaData(getInputFile("most-xml-1.1.xmlc"));
    }

    /**
     * Test 2: Document with empty sections
     */
    public void test2() {
        testParseMetaData(getInputFile("empty-sections.xmlc"));
    }

    /**
     * Test 3: Creating new metadata document
     */
    public void test3() throws XMLCException {
        // setup a new document with some stuff...
        MetaDataDocument doc = MetaDataDocument.newInstance();
        MetaData metaData = doc.getMetaData();
        InputDocument inputDoc = metaData.getInputDocument();
        inputDoc.setProcessSSI(true);
        inputDoc.setDocumentFormat(DocumentFormat.XML);

        // Output and reparse
        File createdFile = getResultFile("new.xmlc");
        doc.serialize(createdFile);
        testParseMetaData(createdFile);
    }

    /**
     * Test 3: Backwards compaibility with xmlc-1.0.xsd
     */
    public void disable_test3() {
        // Disabled due to Xerces not being backwards compatible 
        testParseMetaData(getInputFile("most-xml-1.0.xmlc"));
    }

    /**
     * Test 4: Backwards compaibility with xmlc-1.0.1.xsd
     */
    private void disable_test4() {
        // Disabled due to Xerces not being backwards compatible 
        testParseMetaData(getInputFile("most-xml-1.0.1.xmlc"));
    }
}
