/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: TextNodeEditor.java,v 1.2 2005-01-26 08:29:25 jkjome Exp $
 */

package org.enhydra.xml.xmlc.perf;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
;

/**
 * Editor that edits all text nodes.  Performs a depth-frist traversl
 * of the DOM tree.
 */
public class TextNodeEditor implements DOMEditor {
    /** Has this been run yet? */
    private boolean fHasRun = false;

    public int edit(Document doc) throws Exception {
	Element e = doc.getDocumentElement();
	return edit(e);
    }

    public int edit(Node node) throws Exception {
        int cnt = 0;
	if (node.getNodeType() == Node.TEXT_NODE) {
	    node.setNodeValue("hello");
            cnt++;
	} else {
	    NodeList children = node.getChildNodes();
	    for (int i=0; i<children.getLength(); i++) {
		cnt += edit(children.item(i));
	    }
	}
        return cnt;
    }

    /** Next function, allows support only one edit configuration */
    public boolean next() {
        if (fHasRun) {
            return false;
        } else {
            fHasRun = true;
            return true;
        }
    }


    /** Reset the editor queue. */
    public void reset() {
        fHasRun = false;
    }

    /** Get a description */
    public String toString() {
	return "TextNodeEditor: edits all text nodes in the DOM tree.";
    }
    
}






