/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: BasicTests.java,v 1.2 2005-01-26 08:29:25 jkjome Exp $
 */

package org.enhydra.xml.xmlc.xml;
import java.io.File;
import java.lang.reflect.Method;

import junit.framework.Test;

/**
 * Tests of basic XML functionality.
 */
public class BasicTests extends XmlTestCaseBase {
    /** Factory method to create suite of these tests */
    public static Test suite() {
        return createSuite(BasicTests.class, null);
    }

    /** Constructor */
    public BasicTests(Method method) {
        super(method);
    }

    /** 
     * Test 1: simple doc with no DTD.
     */
    public void test1() {
        XmlBasicTest test
            = new XmlBasicTest(this, getInputFile("simple/tiny.xml"), null);
        test.noValidation();
        test.setUseToDocument();
        test.basicTest();
    }

    /** 
     * Test 2: doc with DTD
     */
    public void test2() {
        XmlBasicTest test
            = new XmlBasicTest(this, getInputFile("xml-examples/6-1.xml"), null);
        test.basicTest();
    }

    /** 
     * Test 3:  Document with lots of entities.
     */
    public void test3() {
        XmlBasicTest test
            = new XmlBasicTest(this, getInputFile("xml-examples/6-17.xml"), null);
        // no text in main doc, so just touch
        test.setEditReloadSource(false);

        test.addXmlExampleCopyEntities();
        test.basicTest();
    }

    /** 
     * Test 4: standalone document
     */
    public void test4() {
        XmlBasicTest test
            = new XmlBasicTest(this, getInputFile("xml-examples/7-3.xml"), null);
        test.setUseToDocument();
        test.basicTest();
    }

    /** 
     * Test 5: big document, the XML specification in XML
     */
    public void test5() {
        XmlBasicTest test
            = new XmlBasicTest(this, getInputFile("xml-spec/REC-xml-20001006.xml"),
                               getInputFile("xml-spec/xmlspec-v21.dtd"));
        test.basicTest();
    }

    /** 
     * Test 6: Comments/processing instructions before root element
     */
    public void test6() {
        XmlBasicTest test
            = new XmlBasicTest(this, getInputFile("bioml/small.bioml"),
                               getInputFile("bioml/bioml.dtd"));
        test.basicTest();
    }

    /** 
     * Test 7: Encoding test
     */
    public void test7() {
        XmlBasicTest test
            = new XmlBasicTest(this, getInputFile("xml-examples/6-17.ascii.xml"),
                               null);
        // no text in main doc, so just touch
        test.setEditReloadSource(false);

        test.addXmlExampleCopyEntities();
        test.setExpectedEncoding("US-ASCII");
        test.basicTest();
    }

    /** 
     * Test 8: Character entity references
     */
    public void test8() {
        XmlBasicTest test
            = new XmlBasicTest(this, getInputFile("simple/charEntities.xml"), null);
        test.basicTest();
    }

    /** 
     * Test 9: Donated by# Chris Webb <chris.webb@voxsurf.com>, showing a
     * large entity reference generating stack overflow in
     * DOMBuilderGenerator.
     */
    public void test9() {
        XmlBasicTest test
            = new XmlBasicTest(this, getInputFile("bigentity/BigEntityTest.vxml"),
                               getInputFile("bigentity/voicexml1-0.dtd"));
        test.addExtraSrcFile(getInputFile("bigentity/BigEntity.ent"));
        test.basicTest();
    }

    /** 
     * Test 10: Test trying to compile a HTML document with a xml parser.
     */
    public void test10() {
        File inputFile = getOtherInputFile("../html", "testPage1.html");
        XmlBasicTest test
            = new XmlBasicTest(this, inputFile, null);
        test.setCompileShouldFail();
        test.basicTest();
    }
}
