/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: MiscTests.java,v 1.3 2006-09-04 05:23:50 jkjome Exp $
 */

package org.enhydra.xml.xmlc.xml;
import java.io.File;
import java.lang.reflect.Method;

import junit.framework.Test;

import org.enhydra.xml.driver.TestException;
import org.enhydra.xml.driver.TestFileOps;
import org.enhydra.xml.xmlc.XMLObject;
import org.enhydra.xml.xmlc.driver.ExecXmlc;
import org.w3c.dom.DocumentType;

/**
 * Various tests that we did put anywhere else.
 */
public class MiscTests extends XmlTestCaseBase {

    /** Factory method to create suite of these tests */
    public static Test suite() {
        return createSuite(MiscTests.class, null);
    }
    
    /** Constructor */
    public MiscTests(Method method) {
        super(method);
    }

    /** 
     * Test 1: Try compile document that needs a xcatalog, but
     * not specifying it.
     */
    public void test1() {
        XmlBasicTest test
            = new XmlBasicTest(this, getInputFile("xcatalog/index.wml"), null);
        test.setCompileShouldFail();
        test.basicTest();
    }

    /** 
     * Test 2: Compile with xcatalog
     */
    public void test2() {
        XmlBasicTest test
            = new XmlBasicTest(this, getInputFile("xcatalog/index.wml"), null);
        //FIXME...
        //NOTE: this nearly works, except that I haven't yet figured out how I
        //can register the classloader containing wml_1.1.test2.xml in the classpath
        //with the org.enhydra.xml.io.XMLEntityResolver.  It works for loading metadata
        //because the XMLCStdFactory/XMLCDeferredParsingFactory accept a classloader to
        //load the document.  The XML parser loads the XMLEntityResolver so there is no
        //opportunity I can yet see to add this classloader.  This issues is unique to
        //our testing strategy.  Usually, the classloader that loads the XMLEntityResolver
        //contains the Entity file to load.
        test.addBoth(ExecXmlc.OPT_XCATALOG,
                     getInputURL("xcatalog/wml.xcat").toExternalForm());
        TestFileOps.copyFileToDir(getInputFile("xcatalog/wml_1.1.test2.xml"),
                                  getClassRoot());
        test.basicTest();
    }

    /** 
     * Test 3: access via URIs: explicit file
     */
    private void disable_test3() {
        //FIXME: disabled, need to figure out how to do this with input mapping
        XmlBasicTest test
            = new XmlBasicTest(this, getInputFile("file:input/xml-examples/6-1.xml"), 
                               getInputFile("xml-examples/document.dtd"));
        test.basicTest();
    }

    /** 
     * Test 4: access via URIs:  ../xxx relative file 
     */
    private void disable_test4() {
        //FIXME: disabled, need to figure out how to do this with input mapping
        XmlBasicTest test
            = new XmlBasicTest(this, getInputFile("file:../xml/input/xml-examples/6-1.xml"), 
                               getInputFile("xml-examples/document.dtd"));
        test.basicTest();
    }

    /** 
     * Test 5: access via URIs:  ../../xxx relative file 
     */
    private void disable_test5() {
        //FIXME: disabled, need to figure out how to do this with input mapping
        XmlBasicTest test
            = new XmlBasicTest(this, 
                               getInputFile("file:../../xmlc/xml/input/xml-examples/6-1.xml"), 
                               getInputFile("../xml/input/xml-examples/document.dtd"));
        test.basicTest();
    }

    /** 
     * Test 6: Compiling SSI file without enabling SSI processing.
     */
    public void test6() {
        XmlBasicTest test
            = new XmlBasicTest(this, getInputFile("bioml/small-ssi.bioml"),
                               getInputFile("bioml/bioml.dtd"));
        test.basicTest();
    }

    /** 
     * Test 7: Compiling with SSI processing.
     */
    public void test7() {
        XmlBasicTest test
            = new XmlBasicTest(this, getInputFile("bioml/small-ssi.bioml"),
                               getInputFile("bioml/bioml.dtd"));
        test.addBoth(ExecXmlc.OPT_SSI);
        test.addExtraSrcFile(getInputFile("bioml/small-ssi.incl"));
        test.basicTest();
    }

    /** 
     * Test 8:
     */
    public void test8() {
        XmlBasicTest test
            = new XmlBasicTest(this, getInputFile("bioml/error-ssi.bioml"),
                               getInputFile("bioml/bioml.dtd"));
        test.setCompileShouldFail();
        test.addBoth(ExecXmlc.OPT_SSI);
        test.basicTest();
    }

    /** 
     * Test 9: Whitespace handling
     */
    public void test9() {
        XmlBasicTest test
            = new XmlBasicTest(this, getInputFile("bioml/space-preserve.bioml"), 
                               getInputFile("bioml/bioml.dtd"));
        test.basicTest();
    }

    /** 
     * Test 10: Bug regression.  This was a bug were the DocumentType was not
     * being generated for LazyDOM, this combined with incorrect resetting of
     * the node ids, results in the compiled-in node ids being out-of-sync
     * with the DOM, and a class-cast exception when an Element was access via
     * it's node id.
     */
    public void test10() throws TestException {
        // compile
        ExecXmlc execXmlc = new ExecXmlc(this);
        execXmlc.addOpt(ExecXmlc.OPT_CLASS, getTestClass());
        execXmlc.addOpt(ExecXmlc.OPT_DEST_DIR, getClassRoot());
        execXmlc.addClassPath(getClassRoot());
        execXmlc.setSrcFile(getInputFile("regression/Error.xml"));
        execXmlc.addExtraSrcFile(getInputFile("regression/AirSent.dtd"));
        if (getParams().getReloading()) {
            execXmlc.addOpt(ExecXmlc.OPT_FOR_RECOMP);
        }
        if (false) {
            //NOTE!!!: if using this, you will need to copy test10.xmlc to the same directory as the output class. See MetaDataDocument.getMetadataOutputFile() for details
            // only needed when debugging.
            execXmlc.addOpt(ExecXmlc.OPT_SOURCE_OUT,
                             getGenSourceDir().getPath());
            execXmlc.addOpt(ExecXmlc.OPT_KEEP);
            execXmlc.addOpt(ExecXmlc.OPT_DUMP);
        }

        File outFile = getResultFile("compile.out");
        File expectFile = getExpectedFile("compile.out");
        execXmlc.compile(outFile);
        getDiffer().diff(expectFile, outFile);

        // Check the document..
        XMLObject doc = loadTestDocument(getTestClass());

        // make sure we have the correct document type.
        String docTypeSystemId = "AirSent.dtd";
        DocumentType docType = doc.getDoctype();
        if (docType == null) {
            throw new TestException("no document type node");
        } else if (!docType.getSystemId().equals(docTypeSystemId)) {
            throw new TestException("Incorrect document type system id: expected \""
                                    + docTypeSystemId + "\", got \"" 
                                    + docType.getSystemId() + "\"");
        }

        // test generated setTextError(String);
        try {
            Method setText = doc.getClass().getMethod("setTextErrorMessage",
                                                      new Class[]{String.class});
            setText.invoke(doc, new Object[]{"game over, man"});
        } catch (Exception except) {
            throw new TestException(except);
        }
    }
}
