/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: XmlBasicTest.java,v 1.2 2005-01-26 08:29:25 jkjome Exp $
 */

package org.enhydra.xml.xmlc.xml;

import java.io.File;

import org.enhydra.xml.xmlc.driver.ExecXmlc;
import org.enhydra.xml.xmlc.driver.XmlcBasicTest;
import org.enhydra.xml.xmlc.driver.XmlcTestCaseBase;

/**
 * Class to run a basic set of XMLC tests on XML files given a set of
 * parameters.  This is extended to implement specific test cases.
 */
public class XmlBasicTest extends XmlcBasicTest {
    /** 
     * Constructor.
     * @param dtdEntity DTD to copy to the directory contain the
     *  document on recompile.  Null if none.
     */
    public XmlBasicTest(XmlTestCaseBase testCase,
                        File inputDoc,
                        File dtdEntity) {
        super(testCase, "xml", inputDoc);
        if (dtdEntity != null) {
            addExtraSrcFile(dtdEntity);
        }
    }

    /**
     * Build a ExecXmlc object with argument required for the initial compile
     * and the recompile/dump
     */
    protected void addStdOpts(ExecXmlc execXmlc) {
    }

    /**
     * Disable validation
     */
    public void noValidation() {
        addBoth(ExecXmlc.OPT_VALIDATE, ExecXmlc.NO);
    }

    /**
     * Add entities used by the input/xml-examples files to tests.
     */
    public void addXmlExampleCopyEntities() {
        XmlcTestCaseBase test = getTestCase();
        addExtraSrcFile(test.getInputFile("xml-examples/address.dtd"));
        addExtraSrcFile(test.getInputFile("xml-examples/body.dtd"));
        addExtraSrcFile(test.getInputFile("xml-examples/body.xml"));
        addExtraSrcFile(test.getInputFile("xml-examples/document.dtd"));
        addExtraSrcFile(test.getInputFile("xml-examples/family.dtd"));
        addExtraSrcFile(test.getInputFile("xml-examples/footer.dtd"));
        addExtraSrcFile(test.getInputFile("xml-examples/footer.xml"));
        addExtraSrcFile(test.getInputFile("xml-examples/header.dtd"));
        addExtraSrcFile(test.getInputFile("xml-examples/header.xml"));
        addExtraSrcFile(test.getInputFile("xml-examples/person.dtd"));
        addExtraSrcFile(test.getInputFile("xml-examples/publisher.dtd"));
    }
}
